# encoding: utf-8

from .coin_agent import CoinAgent

coin_agent = CoinAgent()

def set_proxies(proxies):
    coin_agent.set_proxies(proxies)

# get all coin list
def get_coin_list():
    return coin_agent.get_coin_list()

def get_coin_snapshot(fsym, tsym):
    return coin_agent.get_coin_snapshot(fsym, tsym)

def get_exchange_list():
    return coin_agent.get_exchange_list()

def get_coin_price(fsym, tsym, exchange = "CCCAGG"):
    return coin_agent.get_coin_price(fsym, tsym, exchange)

def get_his_min(fsym, tsym, exchange = "CCCAGG", limit = 2000):
    return coin_agent.get_his_min(fsym, tsym, exchange, limit)

def get_his_hour(fsym, tsym, exchange = "CCCAGG", limit = 2000):
    return coin_agent.get_his_hour(fsym, tsym, exchange, limit)

def get_his_day(fsym, tsym, exchange = "CCCAGG", limit = 2000):
    return coin_agent.get_his_day(fsym, tsym, exchange, limit)



# encoding: utf-8

from .wcup_agent import WorldCupAgent

wcup_agent = WorldCupAgent()

def load_data():
    return wcup_agent.load_data()

def set_proxies(proxies):
    return wcup_agent.set_proxies(proxies)

def get_champion_rank():
    return wcup_agent.get_champion_rank()

def get_finalgame_rank():
    return wcup_agent.get_finalgame_rank()

def get_wingame_rank():
    return wcup_agent.get_wingame_rank()

def get_game_rank():
    return wcup_agent.get_game_rank()

def get_year_rank():
    return wcup_agent.get_year_rank()

def get_champion_fistgame_stat():
    return wcup_agent.get_champion_fistgame_stat()

def get_goal_stat():
    return wcup_agent.get_goal_stat()

def get_champion_goal_stat():
    return wcup_agent.get_champion_goal_stat()


# encoding: utf-8

from opendatatools.common import RestAgent
import json
import pandas as pd

nbs_city_map = {
    '北京':'110000',
    '天津':'120000',
    '石家庄':'130100',
    '唐山':'130200',
    '秦皇岛':'130300',
    '太原':'140100',
    '呼和浩特':'150100',
    '包头':'150200',
    '沈阳':'210100',
    '大连':'210200',
    '丹东':'210600',
    '锦州':'210700',
    '长春':'220100',
    '吉林':'220200',
    '哈尔滨':'230100',
    '牡丹江':'231000',
    '上海':'310000',
    '南京':'320100',
    '无锡':'320200',
    '徐州':'320300',
    '扬州':'321000',
    '杭州':'330100',
    '宁波':'330200',
    '温州':'330300',
    '金华':'330700',
    '合肥':'340100',
    '蚌埠':'340300',
    '安庆':'340800',
    '福州':'350100',
    '厦门':'350200',
    '泉州':'350500',
    '南昌':'360100',
    '九江':'360400',
    '赣州':'360700',
    '济南':'370100',
    '青岛':'370200',
    '烟台':'370600',
    '济宁':'370800',
    '郑州':'410100',
    '洛阳':'410300',
    '平顶山':'410400',
    '武汉':'420100',
    '宜昌':'420500',
    '襄阳':'420600',
    '长沙':'430100',
    '岳阳':'430600',
    '常德':'430700',
    '广州':'440100',
    '韶关':'440200',
    '深圳':'440300',
    '湛江':'440800',
    '惠州':'441300',
    '南宁':'450100',
    '桂林':'450300',
    '北海':'450500',
    '海口':'460100',
    '三亚':'460200',
    '重庆':'500000',
    '成都':'510100',
    '泸州':'510500',
    '南充':'511300',
    '贵阳':'520100',
    '遵义':'520300',
    '昆明':'530100',
    '大理':'532900',
    '西安':'610100',
    '兰州':'620100',
    '西宁':'630100',
    '银川':'640100',
    '乌鲁木齐':'650100',
}

nbs_region_map = {
    '北京'            : '110000',
    '天津'            : '120000',
    '河北省'          : '130000',
    '山西省'          : '140000',
    '内蒙古自治区'    : '150000',
    '辽宁省'          : '210000',
    '吉林省'          : '220000',
    '黑龙江省'        : '230000',
    '上海'            : '310000',
    '江苏省'          : '320000',
    '浙江省'          : '330000',
    '安徽省'          : '340000',
    '福建省'          : '350000',
    '江西省'          : '360000',
    '山东省'          : '370000',
    '河南省'          : '410000',
    '湖北省'          : '420000',
    '湖南省'          : '430000',
    '广东省'          : '440000',
    '广西壮族自治区'  : '450000',
    '海南省'          : '460000',
    '重庆'            : '500000',
    '四川省'          : '510000',
    '贵州省'          : '520000',
    '云南省'          : '530000',
    '西藏自治区'      : '540000',
    '陕西省'          : '610000',
    '甘肃省'          : '620000',
    '青海省'          : '630000',
    '宁夏回族自治区'  : '640000',
    '新疆维吾尔自治区': '650000',
}

nbs_indicator_map_df = {
    # 地方GDP
    'A010101':'地区生产总值_累计值(亿元)',
    'A010103':'地区生产总值指数(上年=100)_累计值(%)',

}

nbs_indicator_map = {

    # 年度GDP
    'A020101':'国民总收入(亿元)',
    'A020102':'国内生产总值(亿元)',
    'A020103':'第一产业增加值(亿元)',
    'A020104':'第二产业增加值(亿元)',
    'A020105':'第三产业增加值(亿元)',
    'A020106':'人均国内生产总值(元)',

    # 人口数量
    'A030101':'年末总人口(万人)',
    'A030102':'男性人口(万人)',
    'A030103':'女性人口(万人)',
    'A030104':'城镇人口(万人)',
    'A030105':'乡村人口(万人)',

    # 人口结构
    'A030301':'年末总人口(万人)',
    'A030302':'0-14岁人口(万人)',
    'A030303':'15-64岁人口(万人)',
    'A030304':'65岁及以上人口(万人)',
    'A030305':'总抚养比(%)',
    'A030306':'少儿抚养比(%)',
    'A030307':'老年抚养比(%)',

    # 70 个大中城市商品房价格情况
    'A010801':'新建住宅销售价格指数(上月=100)',
    'A010802':'新建住宅销售价格指数(上年=100)',
    'A010803':'新建住宅销售价格指数(2015=100)',
    'A010804':'新建商品住宅销售价格指数(上月=100)',
    'A010805':'新建商品住宅销售价格指数(上年=100)',
    'A010806':'新建商品住宅销售价格指数(2015=100)',
    'A010807':'二手住宅销售价格指数(上月=100)',
    'A010808':'二手住宅销售价格指数(上年=100)',
    'A010809':'二手住宅销售价格指数(2015=100)',
    'A01080A':'90平米及以下新建商品住宅销售价格指数(上月=100)',
    'A01080B':'90平米及以下新建商品住宅销售价格指数(上年=100)',
    'A01080C':'90平米及以下新建商品住宅销售价格指数(2015=100)',
    'A01080D':'90-144平米新建商品住宅销售价格指数(上月=100)',
    'A01080E':'90-144平米新建商品住宅销售价格指数(上年=100)',
    'A01080F':'90-144平米新建商品住宅销售价格指数(2015=100)',
    'A01080G':'144平米以上新建商品住宅销售价格指数(上月=100)',
    'A01080H':'144平米以上新建商品住宅销售价格指数(上年=100)',
    'A01080I':'144平米以上新建商品住宅销售价格指数(2015=100)',
    'A01080J':'90平米及以下二手住宅销售价格指数(上月=100)',
    'A01080K':'90平米及以下二手住宅销售价格指数(上年=100)',
    'A01080L':'90平米及以下二手住宅销售价格指数(2015=100)',
    'A01080M':'90-144平米二手住宅销售价格指数(上月=100)',
    'A01080N':'90-144平米二手住宅销售价格指数(上年=100)',
    'A01080O':'90-144平米二手住宅销售价格指数(2015=100)',
    'A01080P':'144平米以上二手住宅销售价格指数(上月=100)',
    'A01080Q':'144平米以上二手住宅销售价格指数(上年=100)',
    'A01080R':'144平米以上二手住宅销售价格指数(2015=100)',

    #CPI 相关
    'A01010101':'居民消费价格指数(上年同月=100)',
    'A01010102':'食品烟酒类居民消费价格指数(上年同月=100)',
    'A01010103':'衣着类居民消费价格指数(上年同月=100)',
    'A01010104':'居住类居民消费价格指数(上年同月=100)',
    'A01010105':'生活用品及服务类居民消费价格指数(上年同月=100)',
    'A01010106':'交通和通信类居民消费价格指数(上年同月=100)',
    'A01010107':'教育文化和娱乐类居民消费价格指数(上年同月=100)',
    'A01010108':'医疗保健类居民消费价格指数(上年同月=100)',
    'A01010109':'其他用品和服务类居民消费价格指数(上年同月=100)',

    # PPI相关
    'A01080101':'工业生产者出厂价格指数(上年同月=100)',
    'A01080102':'生产资料工业生产者出厂价格指数(上年同月=100)',
    'A01080103':'生活资料工业生产者出厂价格指数(上年同月=100)',

    'A010301': '工业生产者购进价格指数(上年同月=100)',
    'A010302': '工业生产者出厂价格指数(上年同月=100)',

    # GDP相关
    'A010101':'国内生产总值_当季值(亿元)',
    'A010102':'国内生产总值_累计值(亿元)',
    'A010103':'第一产业增加值_当季值(亿元)',
    'A010104':'第一产业增加值_累计值(亿元)',
    'A010105':'第二产业增加值_当季值(亿元)',
    'A010106':'第二产业增加值_累计值(亿元)',
    'A010107':'第三产业增加值_当季值(亿元)',
    'A010108':'第三产业增加值_累计值(亿元)',
    'A010109':'农林牧渔业增加值_当季值(亿元)',
    'A01010A':'农林牧渔业增加值_累计值(亿元)',
    'A01010B':'工业增加值_当季值(亿元)',
    'A01010C':'工业增加值_累计值(亿元)',
    'A01010D':'制造业增加值_当季值(亿元)',
    'A01010E':'制造业增加值_累计值(亿元)',
    'A01011D':'建筑业增加值_当季值(亿元)',
    'A01011E':'建筑业增加值_累计值(亿元)',
    'A01011F':'批发和零售业增加值_当季值(亿元)',
    'A01011G':'批发和零售业增加值_累计值(亿元)',
    'A01011H':'交通运输、仓储和邮政业增加值_当季值(亿元)',
    'A01011I':'交通运输、仓储和邮政业增加值_累计值(亿元)',
    'A01011J':'住宿和餐饮业增加值_当季值(亿元)',
    'A01011K':'住宿和餐饮业增加值_累计值(亿元)',
    'A01011L':'金融业增加值_当季值(亿元)',
    'A01011M':'金融业增加值_累计值(亿元)',
    'A01011N':'房地产业增加值_当季值(亿元)',
    'A01011O':'房地产业增加值_累计值(亿元)',
    'A01011P':'信息传输、软件和信息技术服务业增加值_当季值(亿元)',
    'A01011Q':'信息传输、软件和信息技术服务业增加值_累计值(亿元)',
    'A01011R':'租赁和商务服务业增加值_当季值(亿元)',
    'A01011S':'租赁和商务服务业增加值_累计值(亿元)',
    'A01012P':'其他行业增加值_当季值(亿元)',
    'A01012Q':'其他行业增加值_累计值(亿元)',

    # GDP 增速有关
    'A010401':'国内生产总值环比增长速度(%)',

    # M0M1M2相关
    'A1B0101':'货币和准货币(M2)供应量_期末值(亿元)',
    'A1B0102':'货币和准货币(M2)供应量_同比增长(%)',
    'A1B0103':'货币(M1)供应量_期末值(亿元)',
    'A1B0104':'货币(M1)供应量_同比增长(%)',
    'A1B0105':'流通中现金(M0)供应量_期末值(亿元)',
    'A1B0106':'流通中现金(M0)供应量_同比增长(%)',

    # 财政收入
    'A1A0101':'国家财政收入_当期值(亿元)',
    'A1A0102':'国家财政收入_累计值(亿元)',
    'A1A0103':'国家财政收入_累计增长(%)',

    # 财政支出
    'A1A0201':'国家财政支出(不含债务还本)_当期值(亿元)',
    'A1A0202':'国家财政支出(不含债务还本)_累计值(亿元)',
    'A1A0203':'国家财政支出(不含债务还本)_累计增长(%)',

    # 制造业 PMI 相关
    'A190101':'制造业采购经理指数(%)',
    'A190102':'生产指数(%)',
    'A190103':'新订单指数(%)',
    'A190104':'新出口订单指数(%)',
    'A190105':'在手订单指数(%)',
    'A190106':'产成品库存指数(%)',
    'A190107':'采购量指数(%)',
    'A190108':'进口指数(%)',
    'A190109':'出厂价格指数(%)',
    'A190119':'主要原材料购进价格指数(%)',
    'A19011A':'原材料库存指数(%)',
    'A19011B':'从业人员指数(%)',
    'A19011C':'供应商配送时间指数(%)',
    'A19011D':'生产经营活动预期指数(%)',

    # 非织造业PMI
    'A190201':'非制造业商务活动指数(%)',
    'A190202':'新订单指数(%)',
    'A190203':'新出口订单指数(%)',
    'A190204':'在手订单指数(%)',
    'A190205':'存货指数(%)',
    'A190206':'投入品价格指数(%)',
    'A190207':'销售价格指数(%)',
    'A190208':'从业人员指数(%)',
    'A190209':'供应商配送时间指数(%)',
    'A19020A':'业务活动预期指数(%)',

    # 综合PMI
    'A190301':'综合PMI产出指数(%)',

    # 进出口情况
    'A160101':'进出口总值_当期值(千美元)',
    'A160102':'进出口总值_同比增长(%)',
    'A160103':'进出口总值_累计值(千美元)',
    'A160104':'进出口总值_累计增长(%)',
    'A160105':'出口总值_当期值(千美元)',
    'A160106':'出口总值_同比增长(%)',
    'A160107':'出口总值_累计值(千美元)',
    'A160108':'出口总值_累计增长(%)',
    'A160109':'进口总值_当期值(千美元)',
    'A16010A':'进口总值_同比增长(%)',
    'A16010B':'进口总值_累计值(千美元)',
    'A16010C':'进口总值_累计增长(%)',
    'A16010D':'进出口差额_当期值(千美元)',
    'A16010E':'进出口差额_累计值(千美元)',

    # FDI相关
    'A160201':'外商直接投资合同项目数_累计值(个)',
    'A160202':'外商直接投资合同项目数_累计增长(%)',
    'A160203':'合资经营企业外商直接投资合同项目数_累计值(个)',
    'A160204':'合资经营企业外商直接投资合同项目数_累计增长(%)',
    'A160205':'合作经营企业外商直接投资合同项目数_累计值(个)',
    'A160206':'合作经营企业外商直接投资合同项目数_累计增长(%)',
    'A160207':'外资企业外商直接投资合同项目数_累计值(个)',
    'A160208':'外资企业外商直接投资合同项目数_累计增长(%)',
    'A160209':'外商投资股份制企业外商直接投资合同项目数_累计值(个)',
    'A16020A':'外商投资股份制企业外商直接投资合同项目数_累计增长(%)',
    'A16020B':'实际利用外商直接投资金额_累计值(百万美元)',
    'A16020C':'实际利用外商直接投资金额_累计增长(%)',
    'A16020D':'合资经营企业实际利用外商直接投资金额_累计值(百万美元)',
    'A16020E':'合资经营企业实际利用外商直接投资金额_累计增长(%)',
    'A16020F':'合作经营企业实际利用外商直接投资金额_累计值(百万美元)',
    'A16020G':'合作经营企业实际利用外商直接投资金额_累计增长(%)',
    'A16020H':'外资企业实际利用外商直接投资金额_累计值(百万美元)',
    'A16020I':'外资企业实际利用外商直接投资金额_累计增长(%)',
    'A16020J':'外商投资股份制企业实际利用外商直接投资金额_累计值(百万美元)',
    'A16020K':'外商投资股份制企业实际利用外商直接投资金额_累计增长(%)',

    # 社会消费品零售总额
    'A150101':'社会消费品零售总额_当期值(亿元)',
    'A150102':'社会消费品零售总额_累计值(亿元)',
    'A150103':'社会消费品零售总额_同比增长(%)',
    'A150104':'社会消费品零售总额_累计增长(%)',
    'A150105':'限上单位消费品零售额_当期值(亿元)',
    'A150106':'限上单位消费品零售额_累计值(亿元)',
    'A150107':'限上单位消费品零售额_同比增长(%)',
    'A150108':'限上单位消费品零售额_累计增长(%)',

    # 网上零售额
    'A150801':'网上零售额_累计值(亿元)',
    'A150802':'网上零售额_累计增长(%)',
    'A150803':'实物商品网上零售额_累计值(亿元)',
    'A150804':'实物商品网上零售额_累计增长(%)',
    'A150805':'吃类实物商品网上零售额_累计值(亿元)',
    'A150806':'吃类实物商品网上零售额_累计增长(%)',
    'A150807':'穿类实物商品网上零售额_累计值(亿元)',
    'A150808':'穿类实物商品网上零售额_累计增长(%)',
    'A150809':'用类实物商品网上零售额_累计值(亿元)',
    'A150810':'用类实物商品网上零售额_累计增长(%)',

    # 房地产开发投资
    'A140101':'房地产投资_累计值(亿元)',
    'A140102':'房地产投资_累计增长(%)',
    'A140103':'房地产配套工程投资_累计值(亿元)',
    'A140104':'房地产配套工程投资_累计增长(%)',
    'A140105':'房地产住宅投资_累计值(亿元)',
    'A140106':'房地产住宅投资_累计增长(%)',
    'A140107':'90平方米及以下住房投资_累计值(亿元)',
    'A140108':'90平方米及以下住房投资_累计增长(%)',
    'A140109':'144平方米以上住房投资_累计值(亿元)',
    'A14010A':'144平方米以上住房投资_累计增长(%)',
    'A14010B':'别墅、高档公寓投资_累计值(亿元)',
    'A14010C':'别墅、高档公寓投资_累计增长(%)',
    'A14010D':'房地产办公楼投资_累计值(亿元)',
    'A14010E':'房地产办公楼投资_累计增长(%)',
    'A14010F':'房地产商业营业用房投资_累计值(亿元)',
    'A14010G':'房地产商业营业用房投资_累计增长(%)',
    'A14010H':'其它房地产投资_累计值(亿元)',
    'A14010I':'其它房地产投资_累计增长(%)',
    'A14010J':'房地产开发建筑工程投资_累计值(亿元)',
    'A14010K':'房地产开发建筑工程投资_累计增长(%)',
    'A14010L':'房地产开发安装工程投资_累计值(亿元)',
    'A14010M':'房地产开发安装工程投资_累计增长(%)',
    'A14010N':'房地产设备工器具购置投资_累计值(亿元)',
    'A14010O':'房地产设备工器具购置投资_累计增长(%)',
    'A14010P':'房地产其它费用投资_累计值(亿元)',
    'A14010Q':'房地产其它费用投资_累计增长(%)',
    'A14010R':'房地产土地购置费_累计值(亿元)',
    'A14010S':'房地产土地购置费_累计增长(%)',
    'A14010T':'房地产开发计划总投资_累计值(亿元)',
    'A14010U':'房地产开发计划总投资_累计增长(%)',
    'A14010V':'房地产开发新增固定资产投资_累计值(亿元)',
    'A14010W':'房地产开发新增固定资产投资_累计增长(%)',

    # 固定资产投资类
    'A130101':'固定资产投资完成额_累计值(亿元)',
    'A130102':'固定资产投资完成额_累计增长(%)',
    'A130103':'国有及国有控股固定资产投资额_累计值(亿元)',
    'A130104':'国有及国有控股固定资产投资额_累计增长(%)',
    'A130105':'房地产开发投资额_累计值(亿元)',
    'A130106':'房地产开发投资额_累计增长(%)',
    'A130107':'第一产业固定资产投资完成额_累计值(亿元)',
    'A130108':'第一产业固定资产投资完成额_累计增长(%)',
    'A130109':'第二产业固定资产投资完成额_累计值(亿元)',
    'A13010A':'第二产业固定资产投资完成额_累计增长(%)',
    'A13010B':'第三产业固定资产投资完成额_累计值(亿元)',
    'A13010C':'第三产业固定资产投资完成额_累计增长(%)',
    'A13010D':'中央项目固定资产投资完成额_累计值(亿元)',
    'A13010E':'中央项目固定资产投资完成额_累计增长(%)',
    'A13010F':'地方项目固定资产投资完成额_累计值(亿元)',
    'A13010G':'地方项目固定资产投资完成额_累计增长(%)',
    'A13010H':'新建固定资产投资完成额_累计值(亿元)',
    'A13010I':'新建固定资产投资完成额_累计增长(%)',
    'A13010J':'扩建固定资产投资完成额_累计值(亿元)',
    'A13010K':'扩建固定资产投资完成额_累计增长(%)',
    'A13010L':'改建固定资产投资完成额_累计值(亿元)',
    'A13010M':'改建固定资产投资完成额_累计增长(%)',
    'A13010N':'建筑安装工程固定资产投资完成额_累计值(亿元)',
    'A13010O':'建筑安装工程固定资产投资完成额_累计增长(%)',
    'A13010P':'设备工器具购置固定资产投资完成额_累计值(亿元)',
    'A13010Q':'设备工器具购置固定资产投资完成额_累计增长(%)',
    'A13010R':'其他费用固定资产投资完成额_累计值(亿元)',
    'A13010S':'其他费用固定资产投资完成额_累计增长(%)',
    'A13010T':'房屋施工面积_累计值(万平方米)',
    'A13010U':'房屋施工面积_累计增长(%)',
    'A13010V':'房屋竣工面积_累计值(万平方米)',
    'A13010W':'房屋竣工面积_累计增长(%)',
    'A13010X':'新增固定资产_累计值(亿元)',
    'A13010Y':'新增固定资产_累计增长(%)',

}

class NBSAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def prepare_cookies(self, url):
        response = self.do_request(url, None)
        if response is not None:
            cookies = self.get_cookies()
            return cookies
        else:
            return None

    def get_indicator_map(self):
        return pd.DataFrame(list(nbs_indicator_map.items()), columns=['indicator', 'name'])

    def get_region_map(self):
        return pd.DataFrame(list(nbs_region_map.items()), columns=['region', 'name'])

    def get_city_map(self):
        return pd.DataFrame(list(nbs_city_map.items()), columns=['city', 'name'])

    # 获取全国指标
    def _get_qg_indicator(self, cn, category, dbcode = 'hgyd'):
        url = 'http://data.stats.gov.cn/easyquery.htm'
        param = {
            "m": "QueryData",
            "dbcode": dbcode,
            "rowcode": "zb",
            "colcode": "sj",
            "wds" : '[]',
            "dfwds": '[{"wdcode":"sj","valuecode":"LAST36"}, {"wdcode": "zb", "valuecode": "%s"}]' % (category),
        }

        url = 'http://data.stats.gov.cn/easyquery.htm?cn=%s&zb=%s' % (cn, category)
        cookies = self.prepare_cookies(url)

        response = self.do_request(url, param, cookies=cookies)
        rsp = json.loads(response)
        code = rsp['returncode']
        data = rsp['returndata']

        records = data['datanodes']
        result_list = []
        for record in records:
            value = record['data']['data']
            strvalue = record['data']['strdata']
            date  = record['wds'][1]['valuecode']
            indicator = record['wds'][0]['valuecode']

            if indicator in nbs_indicator_map:
                indicator_name = nbs_indicator_map[indicator]
            else:
                indicator_name = ""

            result_list.append({
                "indicator" : indicator,
                "indicator_name" : indicator_name,
                "date"       : date,
                "value"     : value,
                "strvalue"  : strvalue,

            })

        return pd.DataFrame(result_list), ""

    def _get_df_indicator(self, region, cn, category, dbcode = 'fsyd'):
        if region not in nbs_region_map:
            return None, '不合法的省份名称, 请通过get_region_map接口获取正确的省份名称'

        region_code = nbs_region_map[region]
        url = 'http://data.stats.gov.cn/easyquery.htm'

        param = {
            "m": "QueryData",
            "dbcode": dbcode,
            "rowcode": "zb",
            "colcode": "sj",
            "wds" : '[{"wdcode": "region", "valuecode": "%s"}]' % (region_code),
            "dfwds": '[{"wdcode":"zb","valuecode":"%s"}, {"wdcode":"sj","valuecode":"LAST36"}]' % (category),
        }

        url = 'http://data.stats.gov.cn/easyquery.htm?cn=%s&zb=%s&reg=%s' % (cn, category,region_code)
        cookies = self.prepare_cookies(url)

        response = self.do_request(
            url, param,
            cookies=cookies
        )
        rsp = json.loads(response)
        code = rsp['returncode']
        data = rsp['returndata']

        records = data['datanodes']
        result_list = []
        for record in records:
            value = record['data']['data']
            strvalue = record['data']['strdata']
            date  = record['wds'][2]['valuecode']
            indicator = record['wds'][0]['valuecode']

            if indicator in nbs_indicator_map_df:
                indicator_name = nbs_indicator_map_df[indicator]
            elif indicator in nbs_indicator_map:
                indicator_name = nbs_indicator_map[indicator]
            else:
                indicator_name = ""

            result_list.append({
                "region"       : region,
                "indicator" : indicator,
                "indicator_name" : indicator_name,
                "date"       : date,
                "value"     : value,
                "strvalue"  : strvalue,

            })

        return pd.DataFrame(result_list), ""

    def _get_city_indicator(self, city, cn, category, dbcode = 'csyd'):
        if city not in nbs_city_map:
            return None, '不合法的城市名称，请通过get_city_map获取正确的城市名称'

        region_code = nbs_city_map[city]
        url = 'http://data.stats.gov.cn/easyquery.htm'

        param = {
            "m": "QueryData",
            "dbcode": dbcode,
            "rowcode": "zb",
            "colcode": "sj",
            "wds" : '[{"wdcode": "region", "valuecode": %s}]' % (region_code),
            "dfwds": '[{"wdcode":"sj","valuecode":"LAST36"}]',
        }

        url = 'http://data.stats.gov.cn/easyquery.htm?cn=%s&zb=%s&reg=%s' % (cn, category,region_code)
        cookies = self.prepare_cookies(url)

        response = self.do_request(
            url, param,
            cookies=cookies
        )
        rsp = json.loads(response)
        code = rsp['returncode']
        data = rsp['returndata']

        records = data['datanodes']
        result_list = []
        for record in records:
            value = record['data']['data']
            strvalue = record['data']['strdata']
            date  = record['wds'][2]['valuecode']
            indicator = record['wds'][0]['valuecode']

            if indicator in nbs_indicator_map:
                indicator_name = nbs_indicator_map[indicator]
            else:
                indicator_name = ""

            result_list.append({
                "city"       : city,
                "indicator" : indicator,
                "indicator_name" : indicator_name,
                "date"       : date,
                "value"     : value,
                "strvalue"  : strvalue,

            })

        return pd.DataFrame(result_list), ""

    '''
    年度数据
    '''

    # 年度GDP
    def get_gdp_y(self):
        return self._get_qg_indicator('C01', 'A0201', dbcode='hgnd')

    def get_region_gdp_y(self, region):
        return self._get_fs_indicator('E0103', 'A0201', dbcode='fsnd')

    def get_population_size_y(self):
        return self._get_qg_indicator('C01', 'A0301', dbcode='hgnd')

    def get_population_structure_y(self):
        return self._get_qg_indicator('C01', 'A0303', dbcode='hgnd')

    # 70 个大中城市住宅销售价格指数
    def get_house_price_index(self, city):
        return self._get_city_indicator(city, 'E0104', 'A0108', 'csyd')

    def get_cpi(self):
        return self._get_qg_indicator('A01', 'A010101', dbcode = 'hgyd')

    def get_region_cpi(self, region):
        return self._get_df_indicator(region, 'E0101', 'A010101', dbcode = 'fsyd')

    def get_ppi(self):
        return self._get_qg_indicator('A01', 'A010801', dbcode = 'hgyd')

    def get_region_ppi(self, region):
        return self._get_df_indicator(region, 'E0101', 'A0103', dbcode = 'fsyd')

    def get_gdp(self):
        return self._get_qg_indicator('B01', 'A0101', dbcode='hgjd')

    def get_region_gdp(self, region):
        return self._get_df_indicator(region, 'E0102', 'A0101', dbcode='fsjd')

    def get_gdp_q2q(self):
        return self._get_qg_indicator('B01', 'A0104', dbcode = 'hgjd')

    def get_M0_M1_M2(self):
        return self._get_qg_indicator('A01', 'A1B01', dbcode = 'hgyd')

    def get_fiscal_revenue(self):
        return self._get_qg_indicator('A01', 'A1A01', dbcode = 'hgyd')

    def get_fiscal_expend(self):
        return self._get_qg_indicator('A01', 'A1A02', dbcode = 'hgyd')

    def get_manufacturing_pmi(self):
        return self._get_qg_indicator('A01', 'A0B01', dbcode = 'hgyd')

    def get_non_manufacturing_pmi(self):
        return self._get_qg_indicator('A01', 'A0B02', dbcode = 'hgyd')

    def get_pmi(self):
        return self._get_qg_indicator('A01', 'A0B03', dbcode = 'hgyd')

    def get_import_export(self):
        return self._get_qg_indicator('A01', 'A1601', dbcode = 'hgyd')

    def get_fdi(self):
        return self._get_qg_indicator('A01', 'A1602', dbcode = 'hgyd')

    def get_retail_sales(self):
        return self._get_qg_indicator('A01', 'A1501', dbcode = 'hgyd')

    def get_online_retail_sales(self):
        return self._get_qg_indicator('A01', 'A1508', dbcode='hgyd')

    def get_online_retail_sales(self):
        return self._get_qg_indicator('A01', 'A1508', dbcode='hgyd')

    def get_realestate_investment(self):
        return self._get_qg_indicator('A01', 'A1401', dbcode='hgyd')

    def get_region_realestate_investment(self, region):
        return self._get_df_indicator(region, 'E0101', 'A1401', dbcode='fsyd')

    def get_fixed_asset_investment(self):
        return self._get_qg_indicator('A01', 'A1301', dbcode='hgyd')

    def get_region_fixed_asset_investment(self, region):
        return self._get_df_indicator(region, 'E0101', 'A1301', dbcode='fsyd')



# encoding: UTF-8

from .aqi2_agent import AQIStudyAgent

aqistudy_agent = AQIStudyAgent()

def set_proxies(proxies):
    aqistudy_agent.set_proxies(proxies)

def get_city_list():
    return aqistudy_agent.get_city_list()

def get_hist_daily_aqi(city, begindate, enddate):
    return aqistudy_agent.get_hist_daily_aqi(city, begindate, enddate)

def get_daily_hour_aqi(city, date):
    return aqistudy_agent.get_daily_hour_aqi(city, date)

def get_aqi_map(type, timepoint):
    return aqistudy_agent.get_api_map(type, timepoint)




# encoding : utf-8

import base64
from Crypto.Cipher import AES, DES
import random
import hashlib

def pkcs7padding(text):
    """
    明文使用PKCS7填充
    最终调用AES加密方法时，传入的是一个byte数组，要求是16的整数倍，因此需要对明文进行处理
    :param text: 待加密内容(明文)
    :return:
    """
    bs = AES.block_size  # 16
    length = len(text)
    bytes_length = len(bytes(text, encoding='utf-8'))
    # tips：utf-8编码时，英文占1个byte，而中文占3个byte
    padding_size = length if(bytes_length == length) else bytes_length
    padding = bs - padding_size % bs
    # tips：chr(padding)看与其它语言的约定，有的会使用'\0'
    padding_text = chr(padding) * padding
    return text + padding_text

def pkcs7unpadding(text):
    """
    处理使用PKCS7填充过的数据
    :param text: 解密后的字符串
    :return:
    """
    length = len(text)
    unpadding = ord(text[length-1])
    return text[0:length-unpadding]

def aes_encrypt(key, iv, content):
    """
    AES加密
    key,iv使用同一个
    模式cbc
    填充pkcs7
    :param key: 密钥
    :param content: 加密内容
    :return:
    """
    cipher = AES.new(key, AES.MODE_CBC, iv)
    # 处理明文
    content_padding = pkcs7padding(content)
    # 加密
    encrypt_bytes = cipher.encrypt(bytes(content_padding, encoding='utf-8'))
    # 重新编码
    result = str(base64.b64encode(encrypt_bytes), encoding='utf-8')
    return result

def aes_decrypt(key, iv, content):
    """
    AES解密
     key,iv使用同一个
    模式cbc
    去填充pkcs7
    :param key:
    :param content:
    :return:
    """
    cipher = AES.new(key, AES.MODE_CBC, iv)
    # base64解码
    encrypt_bytes = base64.b64decode(content)
    # 解密
    decrypt_bytes = cipher.decrypt(encrypt_bytes)
    # 重新编码
    result = str(decrypt_bytes, encoding='utf8')
    # 去除填充内容
    result = pkcs7unpadding(result)
    return result

def decrypt_response(des_key, des_iv, aes_key, aes_iv, content):
    """
    AES解密
     key,iv使用同一个
    模式cbc
    去填充pkcs7
    :param key:
    :param content:
    :return:
    """
    aes = AES.new(aes_key, AES.MODE_CBC, aes_iv)
    des = DES.new(des_key, DES.MODE_CBC, des_iv)
    # base64解码
    encrypt_bytes = base64.b64decode(content)
    # 解密
    decrypt_bytes = aes.decrypt(encrypt_bytes)
    decrypt_bytes = base64.b64decode(decrypt_bytes)
    decrypt_bytes = des.decrypt(decrypt_bytes)

    #base64解码
    decrypt_bytes = base64.b64decode(decrypt_bytes)
    # 重新编码
    result = str(decrypt_bytes, encoding='utf8')
    # 去除填充内容
    #result = pkcs7unpadding(result)
    return result

'''
his_aes_client_key = "weJGsdsdf6FxF9=="
his_aes_client_iv = "sewg29nsl="
his_des_key = "sgfsfKsg8723jF=="
his_des_iv = "yfw3wexsd="
his_aes_server_key = "efsdsbafa6xFe8lcg=="
his_aes_server_iv = "o2muxyVs5cwedbQ=="

his_aes_client_key = hashlib.md5(his_aes_client_key.encode(encoding="utf8")).hexdigest()[16:32].encode("utf8")
his_aes_client_iv  = hashlib.md5(his_aes_client_iv.encode(encoding="utf8")).hexdigest()[0:16].encode("utf8")

his_des_key = hashlib.md5(his_des_key.encode(encoding="utf8")).hexdigest()[0:8].encode("utf8")
his_des_iv  = hashlib.md5(his_des_iv.encode(encoding="utf8")).hexdigest()[24:32].encode("utf8")

his_aes_server_key = hashlib.md5(his_aes_server_key.encode(encoding="utf8")).hexdigest()[16:32].encode("utf8")
his_aes_server_iv  = hashlib.md5(his_aes_server_iv.encode(encoding="utf8")).hexdigest()[0:16].encode("utf8")
'''

real_aes_server_key = "tGFsbXNwewZlcg=="
real_aes_server_iv = "w9VydmVswewexbQ=="
real_aes_client_key = "WksdsdflFweFZ=="
real_aes_client_iv = "klsfw9nsp="
real_des_key = "ssfefwksdjskdsj=="
real_des_iv = "skzlkpoi="

real_aes_client_key = hashlib.md5(real_aes_client_key.encode(encoding="utf8")).hexdigest()[16:32].encode("utf8")
real_aes_client_iv  = hashlib.md5(real_aes_client_iv.encode(encoding="utf8")).hexdigest()[0:16].encode("utf8")

real_des_key = hashlib.md5(real_des_key.encode(encoding="utf8")).hexdigest()[0:8].encode("utf8")
real_des_iv  = hashlib.md5(real_des_iv.encode(encoding="utf8")).hexdigest()[24:32].encode("utf8")

real_aes_server_key = hashlib.md5(real_aes_server_key.encode(encoding="utf8")).hexdigest()[16:32].encode("utf8")
real_aes_server_iv  = hashlib.md5(real_aes_server_iv.encode(encoding="utf8")).hexdigest()[0:16].encode("utf8")



# encoding: UTF-8

from opendatatools.common import get_current_day
from bs4 import BeautifulSoup
import pandas as pd
import numpy as np

from opendatatools.common import RestAgent
from opendatatools.aqi.constant import city_code_map

class AQIAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def handle_visit_limit(self):
        url = ""

    def get_daily_aqi(self, date):
        url = "http://datacenter.mep.gov.cn/websjzx/report/list.vm"
        page_no = 0
        aqi_result = list()

        while True:
            page_no = page_no + 1
            # 1. 分页爬取数据
            data = {
                'pageNum': page_no,
                'V_DATE': date,
                'xmlname': 1512478367400,
                'roleType': 'CFCD2084',
            }

            rsp = self.do_request(url, data, self.proxies)

            if rsp is None:
                return None

            data = list()
            soup = BeautifulSoup(rsp, "html5lib")
            divs = soup.find_all('div')

            for div in divs:
                if div.has_attr('class') and 'report_main' in div['class']:
                    rows = div.table.findAll('tr')
                    for row in rows:
                        cols = row.findAll('td')
                        if len(cols) == 9:
                            city      = cols[3].text
                            aqi       = cols[4].text
                            indicator = cols[5].text
                            date      = cols[6].text
                            code      = cols[7].text
                            level     = cols[8].text
                            data.append ({
                                "date"  : date,
                                "city"  : city,
                                "aqi"   : aqi,
                                "code"  : code,
                                "level" : level,
                                "indicator" : indicator,
                            })

            if len(data) == 0:
                break;

            aqi_result.extend(data)

        df = pd.DataFrame(aqi_result)
        return df

    def get_hour_aqi(self, time):
        url = "http://datacenter.mep.gov.cn/websjzx/report/list.vm"
        page_no = 0
        aqi_result = list()

        while True:
            page_no = page_no + 1
            # 1. 分页爬取数据
            data = {
                'pageNum': page_no,
                'xmlname': 1512382906122,
                'roleType': 'CFCD2084',
                'V_DATE': time,
                'E_DATE'  : time,
            }

            rsp = self.do_request(url, data, self.proxies)

            if rsp is None:
                return None

            data = list()
            soup = BeautifulSoup(rsp, "html5lib")
            divs = soup.find_all('div')

            for div in divs:
                if div.has_attr('class') and 'report_main' in div['class']:
                    rows = div.table.findAll('tr')
                    for row in rows:
                        cols = row.findAll('td')
                        if len(cols) == 8:
                            city      = cols[2].text
                            aqi       = cols[3].text
                            indicator = cols[4].text
                            time      = cols[5].text
                            code      = cols[6].text
                            level     = cols[7].text
                            data.append ({
                                "time"  : time,
                                "city"  : city,
                                "aqi"   : aqi,
                                "code"  : code,
                                "level" : level,
                                "indicator" : indicator,
                            })

            if len(data) == 0:
                break;

            aqi_result.extend(data)

        df = pd.DataFrame(aqi_result)
        return df

    def get_daily_aqi_onecity(self, city):
        url = 'http://datacenter.mep.gov.cn/websjzx/report/list.vm'
        if city not in city_code_map:
            print("this city is not ready !" + city)
            return None

        city_code = city_code_map[city]

        aqi_result = list()
        page_no = 0

        while True:
            page_no = page_no + 1

            # 1. 分页爬取数据
            data = {
                'pageNum': page_no,
                'citycodes': city_code,
                'citytime': "2000-01-01",
                'xmlname': "1513844769596kqzllb"
            }

            rsp = self.do_request(url, data, self.proxies)

            # 2. 开始解析返回数据，并从html中提取需要的内容
            data = list()
            soup = BeautifulSoup(rsp, "html5lib")
            divs = soup.find_all('div')

            for div in divs:
                if div.has_attr('class') and 'report_main' in div['class']:
                    rows = div.table.findAll('tr')
                    for row in rows:
                        cols = row.findAll('td')
                        if len(cols) == 7:
                            date = cols[1].text
                            aqi  = cols[3].text
                            level = cols[5].text
                            indicator = cols[4].text
                            data.append({
                                "date" : date,
                                "aqi"  : aqi,
                                "level" : level,
                                "indicator" : indicator,
                            })
            if len(data) == 0:
                break;

            aqi_result.extend(data)

        df = pd.DataFrame(aqi_result)
        return df

    def get_recent_daily_aqi_onecity(self, city):
        url = 'http://datacenter.mep.gov.cn/websjzx/report!list.vm'
        if city not in city_code_map:
            print("this city is not ready !" + city)
            return None

        city_code = city_code_map[city]

        data = {
            'citycodes': city_code,
            'citytime': get_current_day(),
            'xmlname': "1513844769596kqzllb"
        }

        rsp = self.do_request(url, data, self.proxies)

        # 2. 开始解析返回数据，并从html中提取需要的内容
        data = list()
        soup = BeautifulSoup(rsp, "html5lib")
        divs = soup.find_all('div')

        for div in divs:
            if div.has_attr('class') and 'report_main' in div['class']:
                rows = div.table.findAll('tr')
                for row in rows:
                    cols = row.findAll('td')
                    if len(cols) == 7:
                        date = cols[1].text
                        aqi  = cols[3].text
                        level = cols[5].text
                        indicator = cols[4].text
                        data.append({
                            "date" : date,
                            "aqi"  : aqi,
                            "level" : level,
                            "indicator" : indicator,
                        })

        df = pd.DataFrame(data)
        return df

    def get_hour_aqi_onecity(self, city, date):
        url = 'http://datacenter.mep.gov.cn/websjzx/report/list.vm'
        if city not in city_code_map:
            print("this city is not ready !" + city)
            return None

        city_code = city_code_map[city]

        aqi_result = list()
        page_no = 0

        while True:
            page_no = page_no + 1

            # 1. 分页爬取数据
            data = {
                'pageNum': page_no,
                'ctiycodes': city_code,
                'citytime': date,
                'xmlname': "1511257916552",
                "queryflag": "close",
                "customquery": "false",
                "isdesignpatterns": "false",
            }

            rsp = self.do_request(url, data, self.proxies)

            # 2. 开始解析返回数据，并从html中提取需要的内容
            data = list()
            soup = BeautifulSoup(rsp, "html5lib")
            divs = soup.find_all('div')

            for div in divs:
                if div.has_attr('class') and 'report_main' in div['class']:
                    rows = div.table.findAll('tr')
                    for row in rows:
                        cols = row.findAll('td')
                        if len(cols) == 7:
                            time = cols[2].text
                            aqi  = cols[4].text
                            city = cols[3].text
                            level = cols[5].text
                            indicator = cols[6].text
                            data.append({
                                "time" : time,
                                "aqi"  : aqi,
                                "city" : city,
                                "level" : level,
                                "indicator" : indicator,
                            })

            aqi_result.extend(data)

            if len(data) < 10:
                break;

        df = pd.DataFrame(aqi_result)
        return df

if __name__ == '__main__':
    aqi = AQIAgent()
    result = aqi.get_hour_aqi_onecity('北京市','2018-05-26')
    print(result)


# encoding: utf-8

from opendatatools.common import RestAgent
from bs4 import BeautifulSoup
import pandas as pd
import json
anjuke_city_map = {
    '北京' : 'bj',
    '上海' : 'sh',
    '杭州' : 'hz',
    '广州' : 'gz',
    '深圳' : 'sz',
    '厦门' : 'xm',
    '苏州' : 'su',
    '重庆' : 'cq',
    '长沙' : 'cs',
    '海口' : 'hk',
    '合肥' : 'hf',
    '济南' : 'jn',
    '青岛' : 'qd',
    '南京' : 'nj',
    '石家庄': 'sjz',
    '沈阳' : 'sy',
    '天津' : 'tj',
    '武汉' : 'wh',
    '无锡' : 'wx',
    '西安' : 'xa',
    '珠海' : 'zh',
    '郑州' : 'zz',
}
class AnjukeAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)
        self.city_map = {}

    def load_city(self):
        url = 'https://www.anjuke.com/sy-city.html'
        response = self.do_request(url)
        if response is None:
            return {}

        soup = BeautifulSoup(response, "html5lib")
        divs = soup.find_all('div')
        for div in divs:
            if div.has_attr('class') and 'letter_city' in div['class']:
                links = div.find_all('a')
                for link in links:
                    url = link['href']
                    city = link.text
                    self.city_map[city] = url

    @staticmethod
    def extract_word(text, start_tag, end_tag):
        index1 = text.find(start_tag)
        index2 = text.find(end_tag, index1)
        return text[index1 + len(start_tag): index2]

    def get_city_list(self):
        if len(self.city_map) == 0:
            self.load_city()

        return self.city_map.keys()

    def get_real_house_price(self, city):

        if len(self.city_map) == 0:
            self.load_city()

        if city not in self.city_map:
            return None, "城市输入错误"

        url = self.city_map[city]
        url_market = url + "/market/"
        response = self.do_request(url_market)
        if response is None:
            return None, '获取数据失败'

        content = AnjukeAgent.extract_word(response, 'drawChart(', ');')
        xyear = json.loads('{' + AnjukeAgent.extract_word(content, 'xyear:{', '},') + '}')
        ydata = json.loads(AnjukeAgent.extract_word(content, 'ydata:', '] '))
        list_date = []
        for month, year in xyear.items():
            month = int(month.replace('月', ''))
            year = int(year.replace('年', ''))
            date = '%04d%02d' % (year, month)
            list_date.append(date)
        list_price = ydata[0]['data']

        df = pd.DataFrame({'date': list_date, 'price': list_price})
        df['city'] = city
        return df, ''

    def get_rent(self, city, page_no):

        url = "https://%s.zu.anjuke.com/p%s/" % (city, page_no)
        response = self.do_request(url, encoding='utf-8')
        soup = BeautifulSoup(response, 'html5lib')
        divs = soup.find_all('div')
        data_list = []
        for div in divs:
            if div.has_attr('class') and 'zu-itemmod' in div['class']:
                try:
                    title = div.find_all('h3')[0].a.text
                    row1 = div.find_all('p')[0].text.split('|')
                    type = row1[0].replace('\n', '').replace(' ', '')
                    area = row1[1].replace('\n', '').replace(' ', '')
                    height = row1[2].split('层')[0]

                    row2 = div.find_all('p')[1].text.replace(' ', '').split('\n')
                    part = row2[1]
                    direction = row2[2]
                    trans = row2[3]
                    price = div.find_all('p')[2].text
                    row3 = div.find_all('address')[0].text.split('\n')

                    addr = row3[1].replace(' ', '')
                    location = row3[2].replace(' ', '').split(r"\\")[0]
                    data_list.append([type, area, height, part, direction, trans, price, addr, location, title])
                except:
                    print('error occurs on this item')
        return data_list, ''





# encoding: utf-8

from .eastmoney_agent import EastMoneyAgent

eastmoney_agent = EastMoneyAgent()

def get_index_list():
    return eastmoney_agent.get_index_list()

def get_index_data(index_code):
    return eastmoney_agent.get_index_data(index_code)




# encoding: utf-8

from opendatatools.common import RestAgent, split_date, date_convert, remove_chinese
import pandas as pd
import json
import zipfile
import io
import re
from xml.etree import ElementTree

SHF_name_map = {"CJ1": "成交量",
                "CJ1_CHG": "成交量增减",
                "PARTICIPANTABBR1": "成交量期货公司",
                "CJ2": "持买仓量",
                "CJ2_CHG": "持买仓量增减",
                "PARTICIPANTABBR2": "持买仓量期货公司",
                "CJ3": "持卖仓量",
                "CJ3_CHG": "持卖仓量增减",
                "PARTICIPANTABBR3": "持卖仓量期货公司",
                "RANK": "名次",
                "INSTRUMENTID": "symbol",
                }


def format_field(x):
    if type(x) == str:
        return x.replace('\n', '').strip()
    else:
        return x


def _merge_df(df_list):
    df_result = None
    for df in df_list:
        if df_result is None:
            df_result = df
        else:
            df_result = pd.merge(df_result, df, left_index=True, right_index=True)
    return df_result


def _concat_df(df_list):
    return pd.concat(df_list)


def _rename_df(df):
    name_map = {
        "会员简称": "期货公司",
        "（手）": "",
    }
    for col in df.columns:
        for name, value in name_map.items():
            if name in col:
                col_new = col.replace(name, value)
                df.rename(columns={col: col_new}, inplace=True)


class SHFAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    # date : %Y%m%d
    def get_trade_rank(self, date):
        url = 'http://www.shfe.com.cn/data/dailydata/kx/pm%s.dat' % date_convert(date, '%Y-%m-%d', '%Y%m%d')
        response = self.do_request(url, None)
        rsp = json.loads(response)

        code = rsp['o_code']
        msg = rsp['o_msg']

        if code != 0:
            return None, msg
        if 'report_date' in rsp.keys():
            date = rsp['report_date']
        else:
            date = date_convert(date, '%Y-%m-%d', "%Y%m%d")
        records = rsp['o_cursor']

        df = pd.DataFrame(records)
        df['date'] = date

        for col in df.columns:
            df[col] = df[col].apply(lambda x: format_field(x))

        df['RANK'] = df['RANK'].apply(lambda x: int(x))
        df = df[(df['RANK'] > 0) & (df['RANK'] <= 20)]
        df.rename(columns=SHF_name_map, inplace=True)
        df = df[list(SHF_name_map.values())]

        return df, ""


class DCEAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    '''
    名次		会员简称	成交量		增减		
    1		海通期货	24,326		9,991		
    2		中信期货	12,926		5,960		
    3		兴证期货	12,835		4,405		
    4		西南期货	11,054		6,614		
    '''

    def _parse_trade_file(self, file, date):
        filename = file.name.encode('cp437').decode('gbk')
        name_items = filename.split("_")
        symbol = name_items[1]

        lines = file.readlines()
        df_list = []

        if date > '2015-12-31':
            charset = 'utf-8'
        else:
            charset = 'gbk'

        for i in range(len(lines)):
            items = lines[i].decode(charset).split()
            if len(items) == 4 and items[0] == '名次':
                head = items
                head[1] = head[2] + head[1]
                head[3] = head[2] + head[3]
                data = []
                for j in range(20):
                    i = i + 1
                    items = lines[i].decode(charset).split()
                    if items[0] == '总计':
                        break
                    data.append(items)
                if data == []:
                    data.append(['', '', '', ''])
                df = pd.DataFrame(data)
                df.columns = head
                df.set_index('名次', inplace=True)
                df_list.append(df)

        df_result = _merge_df(df_list)
        df_result['symbol'] = symbol
        return df_result

    def get_trade_rank(self, date):
        url = 'http://www.dce.com.cn/publicweb/quotesdata/exportMemberDealPosiQuotesBatchData.html'
        year, month, day = split_date(date, '%Y-%m-%d')
        data = {
            "year": year,
            "month": month - 1,  # 脑残程序员的设计
            "day": day,
            "batchExportFlag": 'batch',
        }

        response = self.do_request(url, data, "POST", type='binary')
        zip_ref = zipfile.ZipFile(io.BytesIO(response))
        df_list = []
        for finfo in zip_ref.infolist():
            file = zip_ref.open(finfo, 'r')
            df = self._parse_trade_file(file, date)
            df_list.append(df)

        df_result = _concat_df(df_list)
        df_result['date'] = date
        df_result.reset_index(level=['名次'], inplace=True)

        _rename_df(df_result)

        return df_result, ""


class CZCAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    '''
    品种：苹果AP              日期： 2018-05-30
    名次  |会员简称      |成交量（手）|增减量    |会员简称      |持买仓量  |增减量    |会员简称      |持卖仓量  |增减量
    1     |海通期货      |157,955     |-2,663    |海通期货      |42,771    |3,342     |海通期货      |43,889    |1,843     
    2     |招商期货      |67,527      |-12,527   |华泰期货      |21,091    |-1,093    |华泰期货      |20,927    |-1,991    
    3     |徽商期货      |66,887      |-29,680   |招商期货      |17,302    |-3,063    |招商期货      |20,519    |-2,788    
    4     |光大期货      |66,322      |-10,134   |永安期货      |17,193    |784       |中信期货      |15,779    |716       
    '''

    def _get_url_by_date(self, date):
        year, month, day = split_date(date, '%Y-%m-%d')
        date_int = int(date_convert(date, '%Y-%m-%d', "%Y%m%d"))
        url = 'http://old.czce.com.cn/portal/DFSStaticFiles/Future/%d/%d/FutureDataHolding.txt'
        url_old = 'http://old.czce.com.cn/portal/exchange/%d/datatradeholding/%d.txt'

        if date < '2015-10-01':
            return url_old % (year, date_int)
        else:
            return url % (year, date_int)

    def _get_code(self, text):
        items = re.split("：| |\t|\r\n|", text)
        return items[1]

    def _get_head(self, text):
        items = self._split_field(text)
        items[1] = '成交量' + items[1]
        items[3] = '成交量增减'
        items[4] = '持买仓量' + items[4]
        items[6] = '持买仓量增减'
        items[7] = '持卖仓量' + items[7]
        items[9] = '持卖仓量增减'
        return items

    def _get_head_old(self, text):
        items = ['名次', '成交量期货公司', '成交量', '成交量增减', '持买仓量期货公司', '持买仓量', '持买仓量增减', '持卖仓量期货公司', '持卖仓量', '持卖仓量增减']
        return items

    def get_head(self, text, old):
        if old == True:
            return self._get_head_old(text)
        else:
            return self._get_head(text)

    def _get_data(self, lines, old):
        if old == True:
            sep = ','
        else:
            sep = '|'

        data = []
        for line in lines:
            items = self._split_field(line, splitter=sep)
            data.append(items)
        return data

    def _split_field(self, text, splitter="|"):
        items = text.split(splitter)
        result = [str(x).strip().replace('\r\n', '') for x in items]
        return result

    def _parse_trade_file_old(self, file):
        lines = file.readlines()
        df_list = []
        for i in range(len(lines)):
            items = self._split_field(lines[i], splitter=',')
            if items[0][0:2] == '合约':
                code = self._get_code(lines[i])
                heads = self.get_head(lines[i], old=True)
                a = i
                while True:
                    a += 1
                    items = self._split_field(lines[a], splitter=',')
                    # print(items)
                    if items[0] == '合计':
                        data = self._get_data(lines[i + 1:a + 1], old=True)
                        break
                df = pd.DataFrame(data)
                df.columns = heads
                df['symbol'] = remove_chinese(code)
                df_list.append(df)
        df_result = _concat_df(df_list)
        return df_result

    def _parse_trade_file(self, file):
        lines = file.readlines()
        df_list = []
        for i in range(len(lines)):
            items = self._split_field(lines[i], splitter='|')
            if len(items) == 10 and items[0] == '名次':
                code = self._get_code(lines[i - 1])
                heads = self.get_head(lines[i], old=False)
                a = i
                while True:
                    a += 1
                    items = self._split_field(lines[a], splitter='|')
                    # print(items)
                    if items[0] == '合计':
                        data = self._get_data(lines[i + 1:a + 1], old=False)
                        break
                df = pd.DataFrame(data)
                df.columns = heads
                df['symbol'] = remove_chinese(code)
                df_list.append(df)
        df_result = _concat_df(df_list)
        return df_result

    def parse_trade_file(self, file, date):
        if date < '2015-10-01':
            return self._parse_trade_file_old(file)
        else:
            return self._parse_trade_file(file)

    def get_trade_rank(self, date):
        url = self._get_url_by_date(date)
        response = self.do_request(url, None, "GET", type='binary')
        df = self.parse_trade_file(io.StringIO(response.decode('gbk')), date)
        df['date'] = date
        _rename_df(df)

        return df, ""


class CFEAgent(RestAgent):

    def __init__(self):
        RestAgent.__init__(self)

    def get_trade_rank(self, date):
        if date < '2018-01-01':
            print("CFE网站改版，暂不兼容旧版，目前此接口仅支持2018年以后数据获取")
            return None, '网站改版，暂不兼容'
        products = ['T', 'IF', 'IC', 'IH', 'TF']
        df_list = []
        for product in products:
            df = self._get_trade_rank_by_product(date, product)
            df_list.append(df)

        df = _concat_df(df_list)
        df['date'] = date
        df.reset_index(level=[0, 1], inplace=True)
        return df, ""

    def _get_trade_rank_by_product(self, date, product):
        url = 'http://www.cffex.com.cn/sj/ccpm/%04d%02d/%02d/%s.xml'
        year, month, day = split_date(date, '%Y-%m-%d')
        url = url % (year, month, day, product)
        response = self.do_request(url, None, "GET")
        root = ElementTree.fromstring(response)
        data_list = []
        for dataElements in root:
            if dataElements.tag != 'data':
                continue

            data = {}
            for subElement in dataElements:
                key = subElement.tag
                value = subElement.text
                if key in ['instrumentid', 'datatypeid', 'rank', 'shortname', 'volume', 'varvolume']:
                    data[key] = value
            data_list.append(data)
        df = pd.DataFrame(data_list)

        datatype_map = {
            "0": "成交量",
            "1": "持买单量",
            "2": "持卖单量",
        }

        df_list = []
        for type, name in datatype_map.items():
            df_tmp = df[df['datatypeid'] == type].copy()
            df_tmp['rank'] = df_tmp['rank'].apply(lambda x: int(x))
            df_tmp.rename(columns={"instrumentid": "symbol"}, inplace=True)
            df_tmp.rename(columns={"rank": "名次"}, inplace=True)
            df_tmp.rename(columns={"shortname": name + "期货公司"}, inplace=True)
            df_tmp.rename(columns={"volume": name}, inplace=True)
            df_tmp.rename(columns={"varvolume": name + "增减"}, inplace=True)
            df_tmp.drop(['datatypeid'], axis=1, inplace=True)
            df_tmp.set_index(['symbol', '名次'], inplace=True)
            df_list.append(df_tmp)

        return _merge_df(df_list)


class SinaFuturesAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)
        self.dict_market_map = {
            '沪': 'SHF',
            '连': 'DCE',
            '郑': 'CZC',
            '油': 'INE',
        }

    def convert_market(self, market, product=''):
        if product == '原油':
            return 'INE'
        else:
            return self.dict_market_map[market]

    def _parse_quote_rsp(self, rsp):
        lines = rsp.split('\n')
        list_quotes = []
        for line in lines:
            quote = self._parse_quote_str(line)
            if quote is not None:
                list_quotes.append(quote)

        return pd.DataFrame(list_quotes)

    def _is_cfe_code(self, code):
        code2 = code
        for i in range(10):
            code2 = code2.replace(str(i), '')

        if code2 in ['IF', 'IC', 'IH', 'T', 'TF']:
            return True
        else:
            return False

    def _parse_quote_str(self, line):
        line = line.replace('var hq_str_', '')
        line = line.replace('CFF_RE_', '')

        items = line.split('=')

        if len(items) < 2:
            return None

        code = items[0]
        quote_str = items[1].replace('"', '').replace(';', '')

        fields = quote_str.split(',')

        if self._is_cfe_code(code):
            pass
        else:
            if len(fields) < 18:
                return None

            quote = {
                'code': code,
                'instname': fields[0],
                'time': fields[1],
                'open': fields[2],
                'high': fields[3],
                'low': fields[4],
                'preclose': fields[5],
                'bidprice1': fields[6],
                'askprice1': fields[7],
                'last': fields[8],
                'settle': fields[9],
                'presettle': fields[10],
                'askvol1': fields[11],
                'bidvol1': fields[12],
                'oi': fields[13],
                'volume': fields[14],
                'exchange': self.convert_market(fields[15]),
                'product': fields[16],
                'date': fields[17],
            }

            return quote

    def get_quote(self, codes):
        url = 'http://hq.sinajs.cn/list=%s' % codes
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        df = self._parse_quote_rsp(response)
        return df, ''

    # 1m, 5m, 15m, 30m, 60m, 1d
    def get_kline(self, type, code):

        if self._is_cfe_code(code):

            if type not in ['5m', '15m', '60m', '1d']:
                return None, '不支持的K线类型'

            if type == '1d':
                url = 'http://stock2.finance.sina.com.cn/futures/api/json.php/CffexFuturesService.getCffexFuturesDailyKLine?symbol=%s' % (
                    code)
            else:
                url = 'http://stock2.finance.sina.com.cn/futures/api/json.php/CffexFuturesService.getCffexFuturesMiniKLine%s?symbol=%s' % (
                type, code)
        else:
            if type not in ['5m', '15m', '30m', '60m', '1d']:
                return None, '不支持的K线类型'

            if type == '1d':
                url = 'http://stock2.finance.sina.com.cn/futures/api/json.php/IndexService.getInnerFuturesDailyKLine?symbol=%s' % (
                    code)
            else:
                url = 'http://stock2.finance.sina.com.cn/futures/api/json.php/IndexService.getInnerFuturesMiniKLine%s?symbol=%s' % (
                type, code)

        response = self.do_request(url)
        if response is None or response == 'null':
            return None, '获取数据失败'

        jsonobj = json.loads(response)
        df = pd.DataFrame(jsonobj)
        df.columns = ['datetime', 'open', 'high', 'low', 'close', 'volume']

        return df, ''



# encoding: utf-8

from .chinamoney_agent import ChinaMoneyAgent
from opendatatools.common import date_convert, get_current_day, get_target_date, get_target_date2
import pandas as pd

chinamoney_agent = ChinaMoneyAgent()

def format_date_param(start_date, end_date):
    if end_date is None:
        end_date = get_current_day()

    if start_date is None:
        start_date = get_target_date(-360, '%Y-%m-%d')

    return start_date, end_date

def _split_qry_range(start_date, end_date):
    list_qry_date = []
    p_start_date = start_date
    while True:
        p_end_date   = get_target_date2(p_start_date, 360)
        if p_end_date > end_date:
            p_end_date = end_date
        list_qry_date.append((p_start_date, p_end_date))

        p_start_date = get_target_date2(p_end_date, 1)
        if p_start_date > end_date:
            break

    return list_qry_date

def get_hist_cny_cpr(start_date = None, end_date = None):
    start_date, end_date = format_date_param(start_date, end_date)
    list_qry_date = _split_qry_range(start_date, end_date)
    df_list = []
    for p_start_date, p_end_date in list_qry_date:
        df, msg = chinamoney_agent.get_hist_cny_cpr(p_start_date, p_end_date)
        if df is None:
            return df, msg
        df_list.append(df)

    df_result = pd.concat(df_list)
    df_result.set_index('date', inplace=True)
    df_result.sort_index(inplace=True)

    return df_result, ""

def get_his_shibor(start_date = None, end_date = None):
    start_date, end_date = format_date_param(start_date, end_date)
    list_qry_date = _split_qry_range(start_date, end_date)
    df_list = []
    for p_start_date, p_end_date in list_qry_date:
        df, msg = chinamoney_agent.get_his_shibor(p_start_date, p_end_date)
        if df is None:
            return df, msg
        df_list.append(df)

    df_result = pd.concat(df_list)
    df_result.set_index('showDateCN', inplace=True)
    df_result.sort_index(inplace=True)

    return df_result, ""

def get_realtime_shibor():
    return chinamoney_agent.get_realtime_shibor()

def get_cny_spot_price():
    return chinamoney_agent.get_cny_spot_price()



# encoding: utf-8
"""
    Open Data
    ~~~~
    Open source data tools
    copyright: (c) 2017 quantOS-org.
    license: Apache 2.0, see LICENSE for details.
"""

import os

__version__ = '1.0.1'
SOURCE_ROOT_DIR = os.path.dirname(os.path.abspath(__file__))




# encoding: utf-8

from opendatatools.common import RestAgent
from bs4 import BeautifulSoup
from progressbar import ProgressBar

import pandas as pd
import re

lianjia_city_map = {
    '北京' : 'bj',
    '上海' : 'sh',
    '成都' : 'cd',
    '杭州' : 'hz',
    '广州' : 'gz',
    '深圳' : 'sz',
    '厦门' : 'xm',
    '苏州' : 'su',
    '重庆' : 'cq',
    '长沙' : 'cs',
    '大连' : 'dl',
    '海口' : 'hk',
    '合肥' : 'hf',
    '济南' : 'jn',
    '青岛' : 'qd',
    '南京' : 'nj',
    '石家庄': 'sjz',
    '沈阳' : 'sy',
    '天津' : 'tj',
    '武汉' : 'wh',
    '无锡' : 'wx',
    '西安' : 'xa',
    '烟台' : 'yt',
    '中山' : 'zs',
    '珠海' : 'zh',
    '郑州' : 'zz',
}

class LianjiaAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def get_city_list(self):
        return lianjia_city_map.keys()

    def get_district_by_city(self, city):
        city_code = lianjia_city_map[city]
        url = 'https://%s.lianjia.com/ershoufang/' % city_code
        response = self.do_request(url)

        soup = BeautifulSoup(response, "html5lib")
        divs = soup.find_all('div')

        data_map = {}
        for div in divs:
            if div.has_attr('data-role') and 'ershoufang' in div['data-role']:
                sub_divs = div.find_all('div')
                sub_div  = sub_divs[0]
                links = sub_div.find_all('a')
                for link in links:
                    #link_addr = link
                    #data_list.append(data)
                    key   = link.text
                    value = link['href'].replace('/ershoufang/', '').replace('/', '')
                    data_map[key] = value

        return data_map

    @staticmethod
    def clear_text(text):
        return text.replace('\n', '').strip()

    def get_esf_list(self, city, max_page_no):
        if max_page_no>100:
            max_page_no = 100
        if city in lianjia_city_map:
            city_code = lianjia_city_map[city]
            return self._get_esf_list(city_code, max_page_no)

    def get_esf_list_by_district(self, city, district, max_page_no):
        if max_page_no>100:
            max_page_no = 100

        if city in lianjia_city_map:
            city_code = lianjia_city_map[city]

        district_map = self.get_district_by_city(city)
        if district in district_map:
            district_code = district_map[district]

        return self._get_esf_list_by_district(city_code, district_code, max_page_no)

    def _get_esf_list(self, city_code, max_page_no):
        page_no = 1
        result_list = []
        process_bar = ProgressBar().start(max_value=max_page_no)
        while page_no <= max_page_no:
            process_bar.update(page_no)
            #print('getting data from lianjia.com for page %d' % page_no)
            data_list = self._get_erf_list_url('https://%s.lianjia.com/ershoufang/pg%d/' % (city_code, page_no))
            page_no = page_no + 1
            if (len(data_list) == 0):
                break
            result_list.extend(data_list)

        return pd.DataFrame(result_list)

    def _get_esf_list_by_district(self, city_code, district_code, max_page_no):
        page_no = 1
        result_list = []
        process_bar = ProgressBar().start(max_value=max_page_no)
        while page_no <= max_page_no:
            process_bar.update(page_no)
            #print('getting data from lianjia.com for page %d' % page_no)
            data_list = self._get_erf_list_url('https://%s.lianjia.com/ershoufang/%s/pg%d/' % (city_code, district_code, page_no))
            page_no = page_no + 1
            if (len(data_list) == 0):
                break
            result_list.extend(data_list)

        return pd.DataFrame(result_list)

    def _get_erf_list_url(self, url):
        response = self.do_request(url, None)
        soup = BeautifulSoup(response, "html5lib")
        divs = soup.find_all('div')

        data_list = []
        for div in divs:
            if div.has_attr('class') and 'content' in div['class']:
                sub_divs = div.find_all('div')
                for sub_div in sub_divs:
                    if sub_div.has_attr('class') and 'leftContent' in sub_div['class']:
                        uls = sub_div.find_all('ul')
                        for ul in uls:
                            if ul.has_attr('class') and 'sellListContent' in ul['class']:
                                items = ul.find_all('li')
                                for item in items:
                                    data = self._parse_item_content(item)
                                    data_list.append(data)

        return data_list

    def _parse_item_content(self, item):
        divs = item.find_all('div')
        for div in divs:
            if not div.has_attr('class'):
                continue
            div_class = div['class']

            if  'title' in div_class:
                id = div.a.attrs['data-housecode']
                title = div.a.text
            if  'houseInfo' in div_class:
                spans     = re.split('[/|]', div.text)
                name      = spans[0].strip()
                shape     = spans[1].strip()
                area      = spans[2].strip()
                direction = spans[3].strip()
                quality   = spans[4].strip()
                if (len(spans) >= 6):
                    elevator  = spans[5].strip()
                else:
                    elevator  = ""

            if 'positionInfo' in div_class:
                array = re.split('[)-/]',div.text)
                if len(array) >= 3:
                    floor    = array[0].strip()
                    building = array[1].strip()
                    location = array[2].strip()
                else:
                    floor    = array[0].strip()
                    building = array[0].strip()
                    location = array[1].strip()

            if  'totalPrice' in div_class:
                total_price =  div.text

            if 'unitPrice' in div_class:
                unit_price = div.text

        return {
            'id'         : id,
            "title"      : title,
            "name"       : name,
            "shape"      : shape,
            "area"       : area,
            "direction" : direction,
            "quality"   : quality,
            "elevator"  : elevator,
            "floor"     : floor,
            "building"  : building,
            "location"  : location,
            "total_price" : total_price,
            "unit_price"  : unit_price,
        }




# encoding: UTF-8

import datetime
from .amac_agent import AMACAgent

amac_agent = AMACAgent()

def set_proxies(proxies):
    amac_agent.set_proxies(proxies)

def get_company_list():
    return amac_agent.get_company_list()

def get_company_detail(company_id):
    return amac_agent.get_company_detail(company_id)

def get_fund_list():
    return amac_agent.get_fund_list()

def get_fund_detail(fund_id):
    return amac_agent.get_fund_detail(fund_id)



# encoding: utf-8

import datetime
import pandas as pd
from .stock_agent import SHExAgent, SZExAgent, CSIAgent, XueqiuAgent, SinaAgent, CNInfoAgent, EastMoneyAgent
from opendatatools.common import get_current_day

shex_agent = SHExAgent()
szex_agent = SZExAgent()
csi_agent = CSIAgent()
xq_agent = XueqiuAgent()
sina_agent = SinaAgent()
cninfo_agent = CNInfoAgent()
eastmoney_agent = EastMoneyAgent()

xq_count_map = {
    '1m': -142,
    '5m': -142,
    '15m': -142,
    '30m': -142,
    '60m': -142,
    'day': -142,
}

bar_span_map = {
    '1m': 1,
    '5m': 5,
    '15m': 15,
    '30m': 30,
    '60m': 60,
    'day': 1440,
}


def make_index(period, trade_date):
    bar_index = list()
    span = bar_span_map[period]
    dt = datetime.datetime.strptime(trade_date, '%Y-%m-%d')
    bar_index.extend(
        pd.DatetimeIndex(start="%s 09:30:00" % trade_date, end="%s 11:30:00" % trade_date, freq='%sT' % span)[1:])
    bar_index.extend(
        pd.DatetimeIndex(start="%s 13:00:00" % trade_date, end="%s 15:00:00" % trade_date, freq='%sT' % span)[1:])
    return bar_index


def set_proxies(proxies):
    shex_agent.set_proxies(proxies)
    szex_agent.set_proxies(proxies)
    csi_agent.set_proxies(proxies)
    xq_agent.set_proxies(proxies)


def get_index_list(market='SH'):
    if market == 'SH':
        return shex_agent.get_index_list()

    if market == 'SZ':
        return szex_agent.get_index_list()

    if market == 'CSI':
        return csi_agent.get_index_list()


def get_index_component(symbol):
    temp = symbol.split(".")

    if len(temp) == 2:
        market = temp[1]
        index = temp[0]
        if market == 'SH':
            return shex_agent.get_index_component(index)
        elif market == 'SZ':
            return szex_agent.get_index_component(index)
        elif market == 'CSI':
            return csi_agent.get_index_component(index)
        else:
            return None
    else:
        return None


def get_rzrq_info(market='SH', date=None):
    if date is None:
        date = get_current_day(format='%Y-%m-%d')

    if market == 'SH':
        return shex_agent.get_rzrq_info(date)

    if market == 'SZ':
        return szex_agent.get_rzrq_info(date)

    return None, None


def get_pledge_info(market='SH', date=None):
    if date is None:
        date = get_current_day(format='%Y-%m-%d')

    if market == 'SH':
        return shex_agent.get_pledge_info(date)

    if market == 'SZ':
        return szex_agent.get_pledge_info(date)

    return None, None


def get_dividend(symbol):
    temp = symbol.split(".")

    if len(temp) == 2:
        market = temp[1]
        code = temp[0]
        if market == 'SH':
            return shex_agent.get_dividend(code)
        if market == 'SZ':
            return cninfo_agent.get_dividend(code)


def get_quote(symbols):
    return xq_agent.get_quote(symbols)


def fill_df(df, period, trade_date, symbol):
    df.index = df['time']
    index = make_index(period, trade_date)
    df_new = pd.DataFrame(index=index, columns=['last'])
    df_new['last'] = df['last']
    df_new.fillna(method='ffill', inplace=True)
    df_new['high'] = df['high']
    df_new['low'] = df['low']
    df_new['open'] = df['open']
    df_new.fillna(method='ffill', axis=1, inplace=True)
    df_new['change'] = df['change']
    df_new['percent'] = df['percent']
    df_new['symbol'] = symbol
    df_new['turnover_rate'] = df['turnover_rate']
    df_new['volume'] = df['volume']
    df_new['time'] = df_new.index
    df_new.fillna(0, inplace=True)
    return df_new


# period 1m, 5m, 15m, 30m, 60m, day
def get_kline(symbol, trade_date, period):
    curr_date = datetime.datetime.strptime(trade_date, '%Y-%m-%d')
    next_date = datetime.datetime.strptime(trade_date, '%Y-%m-%d') + datetime.timedelta(days=1)
    timestamp = next_date.timestamp()

    timestamp = int(timestamp * 1000)
    df, msg = xq_agent.get_kline(symbol, timestamp, period, xq_count_map[period])
    if len(df) == 0:
        return df, msg

    df = df[(df.time < next_date) & (df.time >= curr_date)]
    if len(df) < abs(xq_count_map[period]):
        df_new = fill_df(df, period, trade_date, symbol)
        return df_new, ''
    else:
        return df, ''


def get_kline_multisymbol(symbols, trade_date, period):
    symbol_list = symbols.split(',')

    timestamp = datetime.datetime.strptime(trade_date, '%Y-%m-%d').timestamp()
    timestamp = int(timestamp * 1000)
    df, msg = xq_agent.get_kline_multisymbol(symbol_list, timestamp, period, xq_count_map[period])
    next_date = datetime.datetime.strptime(trade_date, '%Y-%m-%d') + datetime.timedelta(days=1)
    if df is None:
        return df, msg

    df = df[df.time < next_date]
    gp = df.groupby('symbol')
    df_list = list()
    for symbol, df_item in gp:
        if len(df_item) < xq_count_map[period]:
            df_list.append(fill_df(df_item, period, trade_date, symbol))
        else:
            df_list(df_item)

    return pd.concat(df_list), ''


def get_timestamp_list(start_date, end_date):
    timestamp_list = []
    curr_date = start_date
    while curr_date <= end_date:
        curr_datetime = datetime.datetime.strptime(curr_date, '%Y-%m-%d')
        timestamp = curr_datetime.timestamp()
        timestamp_list.append(int(timestamp * 1000))
        next_time = curr_datetime + datetime.timedelta(days=1)
        curr_date = datetime.datetime.strftime(next_time, '%Y-%m-%d')

    return timestamp_list


def get_kline_multidate(symbol, start_date, end_date, period):
    timestamp_list = get_timestamp_list(start_date, end_date)
    return xq_agent.get_kline_multitimestamp(symbol, timestamp_list, period, xq_count_map[period])


def get_daily(symbol, start_date, end_date):
    curr_date = start_date
    df_result = []
    while curr_date <= end_date:
        curr_datetime = datetime.datetime.strptime(curr_date, '%Y-%m-%d')
        next_time = curr_datetime + datetime.timedelta(days=100)
        next_date = datetime.datetime.strftime(next_time, '%Y-%m-%d')

        timestamp = curr_datetime.timestamp()
        df, msg = xq_agent.get_kline(symbol, int(timestamp * 1000), 'day', 100)
        if len(df) != 0:
            df_result.append(df[df['time'] < next_time])

        curr_date = next_date

    if len(df_result) > 0:
        df = pd.concat(df_result)
        df = df[(df['time'] >= start_date) & (df['time'] <= end_date)]
        return df, ''
    else:
        return None, '没有获取到数据'


def get_adj_factor(symbol):
    return sina_agent.get_adj_factor(symbol)


def get_trade_detail(symbol, trade_date):
    return sina_agent.get_trade_detail(symbol, trade_date)


def get_report_data(symbol='600000.SH', type='资产负债表'):
    dict_type = {
        '利润表': 'lrb',
        '资产负债表': 'fzb',
        '现金流量表': 'llb',
    }

    if type not in dict_type:
        return None, 'type输入错误，可以输入 %s' % dict_type.keys()

    data = symbol.split(sep='.')
    market = data[1].lower()
    code = data[0]
    return cninfo_agent.get_report_data(market, code, dict_type[type])


def get_shareholder_structure(symbol='600000.SH'):
    data = symbol.split(sep='.')
    market = data[1].lower()
    code = data[0]
    return cninfo_agent.get_shareholder_structure(market, code)


# 单位：百万元
def get_hist_money_flow(symbol):
    data = symbol.split(sep='.')
    market = data[1]
    if market == 'SH':
        marketnum = '1'
    else:
        marketnum = '2'
    code = data[0] + marketnum
    return eastmoney_agent.get_hist_money_flow(code)


# 单位：万元
def get_realtime_money_flow(symbol):
    data = symbol.split(sep='.')
    market = data[1]
    if market == 'SH':
        marketnum = '1'
    else:
        marketnum = '2'
    code = data[0] + marketnum
    return eastmoney_agent.get_realtime_money_flow(code)


# 单位：亿元
def get_realtime_money_flow_market():
    return eastmoney_agent.get_realtime_money_flow_market()


def get_hist_money_flow_market():
    return eastmoney_agent.get_hist_money_flow_market()


def get_allstock_flow():
    return eastmoney_agent.get_allstock_flow()



from .simu_agent import SimuAgent, BarclayAgent, index_map
import pandas as pd

simu_agent = SimuAgent()
barclay_agent = BarclayAgent()

def set_proxies(proxies):
    simu_agent.set_proxies(proxies)
    barclay_agent.set_proxies(proxies)

def login(username, password):
    user_info, msg = simu_agent.login(username, password)
    return user_info, msg

def load_data():
    df_fundlist, msg = simu_agent.load_data()
    return df_fundlist, msg

def get_fund_list():
    return simu_agent.get_fund_list()

def get_fund_nav(fund_id, time_span = 0):
    return simu_agent.get_fund_nav(fund_id, time_span)

def get_barclay_data(index):
    return barclay_agent.get_data(index)

def get_barclay_index_list():
    data_list = []
    for k, v in index_map.items():
        data_list.append([k, v])
    df = pd.DataFrame(data_list, columns=['index', 'prog_cod'])
    return df[['index']], ''


# encoding: utf-8

from .fund_interface import *

__all__ = ['get_fund_list',
           'get_fund_company', 'get_fundlist_by_company',
           'get_fund_type', 'get_fundlist_by_type',
           'get_fund_nav']



# encoding: utf-8

import json
import pandas as pd
import datetime
from opendatatools.common import RestAgent

class EastMoneyAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

        self.index_code = {
            'BDI' : ('EMI00107664', '波罗的海干散货指数'),
            'BPI' : ('EMI00107665', '巴拿马型运费指数'),
            'BCI' : ('EMI00107666', '海岬型运费指数'),
            'BSI' : ('EMI00107667', '超灵便型船运价指数'),
            'BDTI': ('EMI00107668', '原油运输指数'),
            'BCTI': ('EMI00107669', '成品油运输指数'),
        }

    def get_index_list(self):
        index_list = []
        for index, (code, name) in self.index_code.items():
            index_list.append({'index' : index, 'name' : name})

        return pd.DataFrame(index_list), ''

    def _get_index(self, url):
        page_no = 1
        df_list = []
        while True:
            page_url = url + '&p=%d&ps=1000' % page_no
            response = self.do_request(page_url)
            if response is None:
                return None, '获取数据失败'

            jsonobj = json.loads(response)
            df = pd.DataFrame(jsonobj['data'])
            df_list.append(df)

            if len(df) < 1000:
                break

            page_no = page_no + 1

        df_result = pd.concat(df_list)[['DATADATE', 'VALUE']]
        df_result['DATADATE'] = df_result['DATADATE'].apply(lambda x : datetime.datetime.strptime(x,'%Y-%m-%dT%H:%M:%S'))

        return df_result, ''

    def get_index_data(self, index):
        base_url  = "http://dcfm.eastmoney.com/em_mutisvcexpandinterface/api/js/get?type=HYZS_PageStock&token=70f12f2f4f091e459a279469fe49eca5&filter=(ID='%s')&st=DATADATE&js={\"pages\":(tp),\"data\":(x)}"
        index_url = base_url % self.index_code[index][0]
        return self._get_index(index_url)




# encoding: utf-8

from .stock_interface import *

__all__ = [ 'set_proxies',
            'get_index_list', 'get_index_component', 'get_rzrq_info', 'get_dividend',
            'get_quote', 'get_kline',
            'get_kline_multisymbol', 'get_kline_multidate', 'get_daily',
            'get_adj_factor', 'get_trade_detail', 'get_report_data',
            ]



# encoding: utf-8

from opendatatools.common import RestAgent
import json
import pandas as pd
from bs4 import BeautifulSoup
import re

province_map = {
	"北京"   : "10003",
	"天津"   : "10006",
	"河北"   : "10016",
	"山西"   : "10010",
	"内蒙古" : "10002",
	"辽宁"   : "10027",
	"吉林"   : "10004",
	"黑龙江" : "10031",
	"上海"   : "10000",
	"江苏"   : "10014",
	"浙江"   : "10018",
	"安徽"   : "10008",
	"福建"   : "10024",
	"江西"   : "10015",
	"山东"   : "10009",
	"河南"   : "10017",
	"湖北"   : "10021",
	"湖南"   : "10022",
	"广东"   : "10011",
	"广西"   : "10012",
	"海南"   : "10019",
	"重庆"   : "10028",
	"四川"   : "10005",
	"贵州"   : "10026",
	"云南"   : "10001",
	"西藏"   : "10025",
	"陕西"   : "10029",
	"甘肃"   : "10023",
	"青海"   : "10030",
	"宁夏"   : "10007",
	"新疆"   : "10013",
	"台湾"   : "10146",
	"香港"   : "10020",
	"澳门"   : "10145",
}

province_map_inv = {v:k for k,v in province_map.items()}

schooltype_map = {
    "本科" : "10032",
    "专科" : "10033",
}

subject_map = {
	"理科" : "10035",
	"文科" : "10034",
    "综合" : "10166",
}
subject_map_inv = {v:k for k,v in subject_map.items()}

batch_map = {
	"本科批"     : "10154",
	"本科一批"   : "10036",
	"本科二批"   : "10037",
	"本科三批"   : "10038",
	"专科批"     : "10155",
	"本科提前批" : "10149",
    "平行第一段" : "10163",
}

batch_map_inv = {v:k for k,v in batch_map.items()}

class EOLAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def get_score(self, school, province, subject, recruitbatch):
        url = 'https://g.eol.cn/gaokao/school/apitopscorebyyear'
        param = {
            "schoolid" : school,
            "province" : province,
            "type"      : subject,
            "batch"     : recruitbatch,
        }

        response = self.do_request(url, param, method='POST')

        if response is not None:
            rsp = json.loads(response)
            df = pd.DataFrame(rsp)
            print(df)
            return df, ''

        return None, '获取数据失败'

    def get_num_by_page(self, page_no):
        url = 'https://data-gkcx.eol.cn/soudaxue/queryProvinceScore.html'

        param = {
            'messtype' : 'jsonp',
            'lunum' : '1',
            'provinceforschool': '',
            'schooltype' : '',
            'page' : page_no,
            'size' : 10,
            'keyWord' : '',
            'schoolproperty' : '',
            'schoolflag': '',
            'province': '',
            'fstype': '',
            'zhaoshengpici': '',
            'fsyear': '',
        }

        self.add_headers({'Referer' : 'https://gkcx.eol.cn/soudaxue/queryProvinceScoreNum.html'})

        response = self.do_request(url, param=param, method='GET', encoding='utf-8', verify=False)
        if response is not None:
            response = response[5:-2]
            rsp = json.loads(response)
            df = pd.DataFrame(rsp['school'])
            return df, ''

        return None, '获取数据失败'


    def _get_schools(self, province, school_type, subject, page_no):
        url = 'https://g.eol.cn/gaokao/school/listbyareatype/%d?province=%s&schoolType=%s&subject=%s&sort=1'
        url = url % (page_no, province, school_type, subject)

        response = self.do_request(url, None, encoding='utf8')
        data = []

        if response is None:
            return data

        soup = BeautifulSoup(response, "html5lib")
        divs = soup.find_all('div')

        for div in divs:
            if div.has_attr('class') and 'ui-block-a' in div['class'] and 'per45' in div['class']:
                school_id   = div.a['href'].replace('/', '')
                school_name = div.h2.text
                spans = div.p.find_all('span')
                tag_name = ",".join([span.text for span in spans])
                data.append({
                    "school_id"   : school_id,
                    "school_name" : school_name,
                    "tag_name"     : tag_name,
                })

        return data

    def get_schools(self, province, school_type, subject):
        result = []
        page_no = 1
        while True:
            print('getting data from page %d' % page_no)
            data = self._get_schools(province, school_type, subject, page_no)
            page_no = page_no + 1
            result.extend(data)
            if len(data) == 0:
                break

        return result

    def get_school_list(self):

        df_list = []
        for prov_name, province in province_map.items():
            for schooltype_name, school_type in schooltype_map.items():
                for subject_name, subject in subject_map.items():
                    print('working on %s, %s, %s' % (prov_name, schooltype_name, subject_name))
                    data = self.get_schools(province, school_type, subject)
                    if data is not None and len(data) > 0:
                        df = pd.DataFrame(data)
                        df['province']    = prov_name
                        df['school_type'] = schooltype_name
                        df['subject']     =  subject_name
                        df_list.append(df)
                        #print(df)
        result_df = pd.concat(df_list,  ignore_index=True)
        print (result_df)
        return result_df, ''

    def get_score_list_by_school(self, school):
        print ("working on %s" % school)
        df_list = []
        for prov_name, province in province_map.items():
            print ("working on %s" % province_map_inv[province])
            for subject_name, subject in subject_map.items():
                for batch_name, batch in batch_map.items():
                    df, msg = self.get_score(school, province, subject, batch)
                    if df is None:
                        continue
                    #print (df)
                    df['max'] = df['max'].replace('--', 0).replace('', 0)
                    df['min'] = df['min'].replace('--', 0).replace('', 0)
                    df['var'] = df['var'].replace('--', 0).replace('', 0)
                    df['school_id']     = school
                    df['recruit_batch']  = batch_map_inv[batch]
                    df['subject']       = subject_map_inv[subject]
                    df['province']      = province_map_inv[province]
                    df_list.append(df)
        result_df = pd.concat(df_list, ignore_index=True)
        return result_df, ''

    def get_major_list(self):
        url = "http://www.cdgdc.edu.cn/webrms/wwwroot/zgxwyyjsjyxxw/xwyyjsjyxx/xwsytjxx/xk/xkzyml/276559.shtml"
        response = self.do_request(url, None, encoding='gb2312')
        response = response.replace("\n;", '').replace("&nbsp;", '').replace('　', '')
        #print (response)
        data = []

        if response == '':
            return data

        soup = BeautifulSoup(response, "html5lib")
        major_list = []
        for i in range(567):
            result = soup.select("tbody > tr")[i]
            text = result.get_text()
            if text != '':
                major_list.append(text)
        return major_list

    def get_major(self):
        major_list  = self.get_major_list()
        majors      = []
        first_level = []
        category    = []
        for item in major_list:
            if ('学科门类' not in item) & (item[-1] != '类'):
                majors.append(item)
            elif (item[-1] == '类'):
                first_level.append(item)
            else:
                category.append(item)
        return category, first_level, majors

    def get_score_major_by_school_flag_and_page(self, school_flag, page_no):
        url = "https://data-gkcx.eol.cn/soudaxue/querySpecialtyScore.html"
        param = {
            'messtype': 'jsonp',
            'provinceforschool': '',
            'schooltype':'',
            'page': page_no,
            'size': 50,
            'keyWord': '',
            'schoolproperty': '',
            'schoolflag': school_flag,
            'province': '',
            'fstype': '',
            'zhaoshengpici': '',
            'fsyear': '',
        }

        self.add_headers({'Referer': 'https://gkcx.eol.cn/soudaxue/querySpecialtyScore.html'})

        response = self.do_request(url, param=param, method='GET', encoding='utf-8', verify=False)
        if response is not None:
            response = response[5:-2]
            rsp = json.loads(response)
            df = pd.DataFrame(rsp['school'])[['schoolid', 'schoolname', 'specialtyname', 'localprovince', 'studenttype', 'year', 'batch', 'max', 'min', 'var']]
            df['max'] = df['max'].replace('--', 0).replace('', 0)
            df['min'] = df['min'].replace('--', 0).replace('', 0)
            df['var'] = df['var'].replace('--', 0).replace('', 0)
            num = rsp['totalRecord']['num']
            return df, num, ''

        return None, '', '获取数据失败'

    def get_major_rank(self, xid):
        url = 'http://souky.eol.cn/assess_result.php'
        param = {
            'xid': xid,
            'flag': 1,
        }

        response = self.do_request(url, param=param, method='GET', encoding='gbk')
        if response is not None:
            rsp = json.loads(response)
            return rsp, ''

        return None, '获取数据失败'

    def get_batch_scores(self):
        url = 'http://www.eol.cn/html/g/fsx/index.shtml'
        response = self.do_request(url, encoding='gb2312')
        soup = BeautifulSoup(response, "html5lib")

        provinces = soup.find_all("div", class_ = "area-name")
        province_list = []
        for province in provinces:
            province_list.append(province.text)

        score_list = []
        scores = soup.find_all("div", attrs={"class" : 'area-table-item'})
        for score in scores:
            if len(score['class']) == 1:
                temp = []
                for item in score.select("table tbody tr td"):
                    temp.append(item.text)
                score_list.append(temp)
        df = pd.DataFrame(score_list)
        df['province'] = province_list
        df.set_index('province', inplace=True)
        return df, ''

    def get_batch_scores_hist(self):
        url = 'http://www.eol.cn/html/g/fsx/index.shtml'
        response = self.do_request(url, encoding='gb2312')
        soup = BeautifulSoup(response, "html5lib")

        provinces = soup.find_all("div", class_="area-name")
        province_list = []
        for province in provinces:
            province_list.append(province.text)

        score_list = []
        scores = soup.find_all("div", attrs={"class": 'area-table-item'})
        i = 0
        for score in scores:
            if len(score['class']) == 1:
                province = province_list[i]
                j = -1
                i += 1
            else:
                temp = [province, 2017-j % 5]
                for item in score.select("table tbody tr td"):
                    temp.append(item.text)
                score_list.append(temp)
            j += 1
        df = pd.DataFrame(score_list)
        df.set_index(0, inplace=True)
        return df, ''

    def get_batch_score_hist_break(self):
        url = "https://data-gkcx.eol.cn/soudaxue/queryProvince.html"
        param = {
            'messtype': 'jsonp',
            'luqutype3':'',
            'province3':'',
            'year3':'',
            'luqupici3':'',
            'page': 1,
            'size': 10,
        }
        response = self.do_request(url, param=param, method='GET', verify=False)
        print (response)

if __name__ == "__main__":

    '''
    school_list = [
        '102',
        '104',
        '105',
        '107',
        '109',
        '111',
        '114',
        '119',
        '1218',
        '1219',
        '122',
        '123',
        '125',
        '126',
        '127',
        '131',
        '132',
        '134',
        '138',
        '140',
        '143',
        '1457',
        '31',
        '3255',
        '330',
        '332',
        '34',
        '42',
        '44',
        '45',
        '46',
        '47',
        '52',
        '557',
        '59',
        '60',
        '61',
        '66',
        '661',
        '73',
        '939',
        '97',
        '99',
    ]
    '''

    import time
    agent = EOLAgent()
    df_list = []
    for school in  ['31', '52', '99', '661', '939', '1219']:#school_list:
        df, msg = agent.get_score(school, '10018', '10166', '10163')
        df['year'] = df['year'].apply(lambda x : str(x))
        df = df[df['year'] == '2017']
        df['school_id'] = school
        df_list.append(df)
        time.sleep(1)

    df = pd.concat(df_list)
    df.to_csv('1.csv')



from opendatatools.common import RestAgent, md5
from progressbar import ProgressBar
import json
import pandas as pd
import io
import hashlib
import time

index_map = {
    'Barclay_Hedge_Fund_Index'      : 'ghsndx',
    'Convertible_Arbitrage_Index'   : 'ghsca',
    'Distressed_Securities_Index'   : 'ghsds',
    'Emerging_Markets_Index'        : 'ghsem',
    'Equity_Long_Bias_Index'        : 'ghselb',
    'Equity_Long_Short_Index'       : 'ghsels',
    'Equity_Market_Neutral_Index'   : 'ghsemn',
    'European_Equities_Index'       : 'ghsee',
    'Event_Driven_Index'            : 'ghsed',
    'Fixed_Income_Arbitrage_Index'  : 'ghsfia',
    'Fund_of_Funds_Index'           : 'ghsfof',
    'Global_Macro_Index'            : 'ghsmc',
    'Healthcare_&_Biotechnology_Index': 'ghsbio',
    'Merger_Arbitrage_Index'        : 'ghsma',
    'Multi_Strategy_Index'          : 'ghsms',
    'Pacific_Rim_Equities_Index'    : 'ghspre',
    'Technology_Index'              : 'ghstec',
}


class SimuAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)
        self.user_info = None
        self.df_fundlist = None
        self.cookies = None

    def login(self, username, password):
        url = 'https://passport.simuwang.com/index.php?m=Passport&c=auth&a=login&type=login&name=%s&pass=%s&reme=1&rn=1' % (username, password)
        self.add_headers({'Referer': 'https://dc.simuwang.com/'})
        response = self.do_request(url)
        if response is None:
            return None, '登录失败'

        jsonobj = json.loads(response)
        suc = jsonobj['suc']
        msg = jsonobj['msg']

        if suc != 1:
            return None, msg

        self.cookies = self.get_cookies()

        self.user_info = jsonobj['data']
        return self.user_info, msg

    def prepare_cookies(self, url):
        response = self.do_request(url, None)
        if response is not None:
            cookies = self.get_cookies()
            return cookies
        else:
            return None

    def _get_rz_token(self, time):
        mk = time * 158995555893
        mtoken = md5(md5(str(mk))) + '.' + str(time)
        return mtoken

    def _get_fund_list_page(self, page_no):
        url = 'https://dc.simuwang.com/ranking/get?page=%s&condition=fund_type:1,6,4,3,8,2;ret:9;rating_year:1;istiered:0;company_type:1;sort_name:profit_col2;sort_asc:desc;keyword:' % page_no
        response = self.do_request(url)

        if response is None:
            return None, '获取数据失败', None

        jsonobj = json.loads(response)

        code = jsonobj['code']
        msg  = jsonobj['msg']

        if code != 1000:
            return None, msg, None

        df = pd.DataFrame(jsonobj['data'])
        pageinfo = jsonobj['pager']
        return df, '', pageinfo

    def load_data(self):
        page_no = 1
        df_list = []
        df, msg, pageinfo = self._get_fund_list_page(page_no)
        if df is None:
            return None, msg

        df_list.append(df)
        page_count = pageinfo['pagecount']
        process_bar = ProgressBar().start(max_value=page_count)

        page_no = page_no + 1
        while page_no <= page_count:
            df, msg, pageinfo = self._get_fund_list_page(page_no)
            if df is None:
                return None, msg
            df_list.append(df)
            process_bar.update(page_no)
            page_no = page_no + 1

        self.df_fundlist = pd.concat(df_list)
        return self.df_fundlist, ''

    def get_fund_list(self):
        if self.df_fundlist is None:
            return None, '请先加载数据 load_data'

        return self.df_fundlist, ''

    def _get_sign(self, url, params):
        str = url
        for k,v in params.items():
            str = str + k + params[k]

        sha1 = hashlib.sha1()
        sha1.update(str.encode('utf8'))
        sign = sha1.hexdigest()

        return sign

    def _get_token(self, fund_id):
        sign = self._get_sign('https://dc.simuwang.com/Api/getToken', {'id' : fund_id})
        url = 'https://dc.simuwang.com/Api/getToken?id=%s&sign=%s' % (fund_id, sign)
        self.add_headers({'Referer': 'https://dc.simuwang.com/'})
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        jsonobj = json.loads(response)
        code = jsonobj['code']
        msg  = jsonobj['message']

        if code != 1000 :
            return code, msg

        self.cookies.update(self.get_cookies())

        salt = jsonobj['data']

        muid = self.user_info['userid']
        #str = 'id%smuid%spage%s%s' % (fund_id, muid, page_no, salt)
        str = '%s%s' % (fund_id, salt)
        sha1 = hashlib.sha1()
        sha1.update(str.encode('utf8'))
        token = sha1.hexdigest()

        return token, ''

    def _get_fund_nav_page(self, fund_id, page_no):
        muid = self.user_info['userid']
        token, msg = self._get_token(fund_id)
        if token is None:
            return None, '获取token失败: ' + msg, ''

        url = 'https://dc.simuwang.com/fund/getNavList.html'
        self.add_headers({'Referer': 'https://dc.simuwang.com/product/%s.html' % fund_id})
        data = {
            'id'   : fund_id,
            'muid' : muid,
            'page' : str(page_no),
            'token': token,
        }
        response = self.do_request(url, param=data, cookies=self.cookies, encoding="utf8")
        if response is None:
            return None, '获取数据失败', ''

        jsonobj = json.loads(response)
        code = jsonobj['code']
        msg  = jsonobj['msg']

        if code != 1000 :
            return code, msg, ''

        df = pd.DataFrame(jsonobj['data'])
        pageinfo = jsonobj['pager']
        return df, '', pageinfo

    def _bit_encrypt(self, str, key):
        cryText = ''
        keyLen = len(key)
        strLen = len(str)
        for i in range(strLen):
            k = i % keyLen
            cryText = cryText + chr(ord(str[i]) - k)

        return cryText

    def _bit_encrypt2(self, str, key):
        cryText = ''
        keyLen = len(key)
        strLen = len(str)
        for i in range(strLen):
            k = i % keyLen
            cryText = cryText + chr(ord(str[i]) ^ ord(key[k]))

        return cryText

    def _decrypt_data(self, str, func, key):
        # return self._bit_encrypt(str, 'cd0a8bee4c6b2f8a91ad5538dde2eb34')
        # return self._bit_encrypt(str, '937ab03370497f2b4e8d0599ad25c44c')
        # return self._bit_encrypt(str, '083975ce19392492bbccff21a52f1ace')
        return func(str, key)

    def _get_decrypt_info(self, fund_id):
        url = 'https://dc.simuwang.com/product/%s.html' % fund_id
        response = self.do_request(url, param=None, cookies=self.cookies, encoding="utf8")
        if response is None:
            return None, '获取数据失败', ''

        if "String.fromCharCode(str.charCodeAt(i) - k)" in response:
            decrypt_func = self._bit_encrypt
        else:
            decrypt_func = self._bit_encrypt2

        if response.find("return xOrEncrypt(str, ")> 0:
            tag = "return xOrEncrypt(str, "
        else:
            tag = "return bitEncrypt(str, "
        pos = response.index(tag) + len(tag) + 1
        key = response[pos:pos+32]
        return decrypt_func, key

    def get_fund_nav(self, fund_id, time_elapse = 0):

        if self.user_info is None:
            return None, '请先登录'

        page_no = 1
        df_list = []
        df, msg, pageinfo = self._get_fund_nav_page(fund_id, page_no)
        if df is None:
            return None, msg

        df_list.append(df)
        page_count = pageinfo['pagecount']

        page_no = page_no + 1
        while page_no <= page_count:
            try_times = 1
            while try_times <= 3:
                df, msg, pageinfo = self._get_fund_nav_page(fund_id, page_no)
                if df is None:
                    if  try_times > 3:
                        return None, msg
                    else:
                        try_times = try_times + 1
                        continue
                else:
                    df_list.append(df)
                    break
            page_no = page_no + 1
            if time_elapse > 0:
                time.sleep(time_elapse)

        df_nav = pd.concat(df_list)
        df_nav.drop('c', axis=1, inplace=True)
        df_nav.rename(columns={'d': 'date', 'n': 'nav', 'cn' : 'accu_nav', 'cnw' : 'accu_nav_w'}, inplace=True)

        # 这个网站搞了太多的小坑
        func, key = self._get_decrypt_info(fund_id)
        df_nav['nav']         = df_nav['nav'].apply(lambda x : self._decrypt_data(x, func, key))
        df_nav['accu_nav']   = df_nav['accu_nav'].apply(lambda x : self._decrypt_data(x, func, key))
        df_nav['accu_nav_w'] = df_nav['accu_nav_w'].apply(lambda x : self._decrypt_data(x, func, key))
        #df_nav['nav'] = df_nav['nav'] - df_nav.index * 0.01 - 0.01
        #df_nav['accu_nav'] = df_nav['accu_nav'].apply(lambda x: float(x) - 0.01)
        #df_nav['accu_nav_w'] = df_nav['accu_nav_w'].apply(lambda x: float(x) - 0.02)

        return df_nav, ''

class BarclayAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)
        self.add_headers({'Referer': 'https://www.barclayhedge.com/research/indices/ghs/Equity_Long_Short_Index.html'})
        self.add_headers({'Content - Type': 'application / x - www - form - urlencoded'})

    def get_data(self, index):
        prog_cod = index_map[index]
        url = "https://www.barclayhedge.com/cgi-bin/barclay_stats/ghsndx.cgi"
        param = {
            'dump': 'excel',
            'prog_cod': prog_cod,
            }
        response = self.do_request(url, param=param, method='POST', type='binary')
        if response is not None:
            excel = pd.ExcelFile(io.BytesIO(response))
            df = excel.parse('Sheet1').dropna(how='all').copy().reset_index().drop(0)
            df.columns = ['year', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec', 'YTD']
            df = df.set_index('year')
            return df, ''

        return None, "获取数据失败"


# encoding: utf-8

from opendatatools.common import RestAgent
import json
import datetime
import io
import pandas as pd

class XuangubaoAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)
        self.token = None
        self.add_headers({
            "x-appgo-platform" : "device=pc",
            "referer" : "https://xuangubao.cn/zhutiku",
            "origin": "https://xuangubao.cn",
        })

    def login(self, username, password):
        url = "https://api.xuangubao.cn/api/account/mobile_login"
        data = {
            'Mobile'   : username,
            'Password' : password,
        }

        response = self.do_request(url, param = json.dumps(data), method="POST")
        if response is not None:
            rsp = json.loads(response)
            self.token = rsp["Token"]
            self.add_headers({
                "token" : self.token
            })
            return True
        else:
            return False

    def get_theme(self):
        url = "https://flash-api.xuangubao.cn/api/plate/rank?count=10000&is_asc=true&field=core_avg_pcp"
        response = self.do_request(url, method="GET")
        if response is not None:
            rsp = json.loads(response)
            data = rsp["data"]
            plates = ",".join([str(x) for x in data])
            url_info = "https://flash-api.xuangubao.cn/api/plate/data?fields=core_avg_pcp,plate_name,plate_id&plates=%s" % plates

            response_info = self.do_request(url_info, method="GET")
            if response_info is not None:
                rsp_info = json.loads(response_info)
                info = rsp_info["data"]
                df = pd.DataFrame(info).T
                df.columns = ["core_avg_pcp", "plate_id", "plate_name"]
                return df, ""
        else:
            return None, "获取数据失败"

    def get_theme_stock(self, tid):
        url = "https://flash-api.xuangubao.cn/api/plate/plate_set?id=%s" % tid
        self.add_headers({
            "X-Appgo-Token" : self.token
        })
        response = self.do_request(url, method="GET")
        if response is not None:
            rsp = json.loads(response)
            stocks = rsp["data"]['stocks']
            df = pd.DataFrame(stocks)
            return df, ""
        else:
            return None, "获取数据失败"

    def get_theme_kline(self, tid):
        url = "https://flash-api.xuangubao.cn/api/plate/index_history?plate_id=%s&index_type=1&data_count=10000" % tid
        response = self.do_request(url, method="GET")
        if response is not None:
            rsp = json.loads(response)
            data = rsp["data"]
            df = pd.DataFrame(data)
            df["date_time"] = df["date_time"].apply(lambda x: datetime.datetime.fromtimestamp(x).strftime("%Y-%m-%d"))
            return df, ""
        else:
            return None, "获取数据失败"

    def get_theme_event_onepage(self, tid, tail):
        url = "https://api.xuangubao.cn/api/pc/bkjMsgs/%d?limit=50&TailMark=%d" % (tid, tail)
        response = self.do_request(url, method="GET")
        if response is not None:
            rsp = json.loads(response)
            data = rsp["Messages"]
            TailMark = int(rsp["TailMark"])
            df = pd.DataFrame(data)
            if len(df) > 0:
                return df, TailMark

        return None, 0

    def get_theme_event(self, tid):
        df_list = []
        tail = 0
        while True:
            df, tail = self.get_theme_event_onepage(tid, tail)
            if df is not None:
                df_list.append(df)
            else:
                break

        if len(df_list)>1:
            df = pd.concat(df_list)
            df["CreatedAt"] = df["CreatedAt"].apply(lambda x : datetime.datetime.fromtimestamp(x))
            df["UpdatedAt"] = df["UpdatedAt"].apply(lambda x : datetime.datetime.fromtimestamp(x))
            df["ManualUpdatedAt"] = df["ManualUpdatedAt"].apply(lambda x : datetime.datetime.fromtimestamp(x))
            df = df.set_index("Id")
            return df, ""

        return None, "获取数据失败"



# -*- coding: UTF-8 -*-

import sys, time


class ShowProcess():
    """
    显示处理进度的类
    调用该类相关函数即可实现处理进度的显示
    """
    i = 0  # 当前的处理进度
    max_steps = 0  # 总共需要处理的次数
    max_arrow = 50  # 进度条的长度
    infoDone = 'done'

    # 初始化函数，需要知道总共的处理次数
    def __init__(self, max_steps, infoDone='Done'):
        self.max_steps = max_steps
        self.i = 0
        self.infoDone = infoDone

    # 显示函数，根据当前的处理进度i显示进度
    # 效果为[>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>]100.00%
    def show_process(self, i=None):
        if i is not None:
            self.i = i
        else:
            self.i += 1
        num_arrow = int(self.i * self.max_arrow / self.max_steps)  # 计算显示多少个'>'
        num_line = self.max_arrow - num_arrow  # 计算显示多少个'-'
        percent = self.i * 100.0 / self.max_steps  # 计算完成进度，格式为xx.xx%
        process_bar = '[' + '>' * num_arrow + '-' * num_line + ']' \
                      + '%.2f' % percent + '%' + '\r'  # 带输出的字符串，'\r'表示不换行回到最左边
        print(process_bar, end="", flush=True)  # 打印字符到终端
        if self.i >= self.max_steps:
            self.close()

    def close(self):
        print('')
        print(self.infoDone)
        self.i = 0


if __name__ == '__main__':
    max_steps = 100

    process_bar = ShowProcess(max_steps, 'OK')

    for i in range(max_steps):
        process_bar.show_process()
        time.sleep(0.1)



# encoding: utf-8

from .movie_agent import MovieAgent

movie_agent = MovieAgent()

# 总票房排行
def get_boxoffice_rank():
    return movie_agent.get_boxoffice()

# 实时票房排名
def get_realtime_boxoffice():
    return movie_agent.get_realtime_boxoffice()

# 单日票房排行(近7日)
def get_recent_boxoffice(day):
    return movie_agent.get_recent_boxoffice(day)

# 单月票房统计
def get_monthly_boxoffice(date):
    return movie_agent.get_monthly_boxoffice(date)

# 年度票房统计
def get_yearly_boxoffice(year):
    return movie_agent.get_yearly_boxoffice(year)




# encoding: utf-8

from opendatatools.common import RestAgent
import pandas as pd
import io
import datetime

class USStockAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def prepare_cookies(self, url):
        response = self.do_request(url, None)
        if response is not None:
            cookies = self.get_cookies()

            # "CrumbStore":{"crumb":"JSkGHd8NKBu"}
            key = "CrumbStore"
            pos = response.find(key)
            if pos != -1:
                idx1 = response.find('{', pos)
                idx2 = response.find('}', pos)
                crumb = response[idx1+1 : idx2].split(':')[1].replace('"', '')
                return cookies, crumb
        else:
            return None, ''

    def _get_symbols(self, market):
        url = 'https://www.nasdaq.com/screening/companies-by-name.aspx?letter=0&exchange=%s&render=download' % market
        response = self.do_request(url, None, method='GET', type='binary')
        if response is not None:
            df = pd.read_csv(io.BytesIO(response))
            return df
        return None

    def get_symbols(self):
        df_nasdaq = self._get_symbols('nasdaq')
        df_nyse   = self._get_symbols('nyse')
        df_amex   = self._get_symbols('amex')

        return pd.concat([df_nasdaq, df_nyse, df_amex]), ''

    def _get_data(self, symbol, events):

        url_cookies = 'https://finance.yahoo.com/quote/%s/history?p=%s' % (symbol, symbol)
        cookies, crumb = self.prepare_cookies(url_cookies)

        now = datetime.datetime.now().timestamp()
        url = 'https://query1.finance.yahoo.com/v7/finance/download/%s?period1=%d&period2=%d&interval=1d&events=%s&crumb=%s' % (symbol, 0, int(now), events, crumb)
        self.add_headers({'referer' : url_cookies})
        response = self.do_request(url, None, method='GET', type='binary', cookies=cookies)
        if response is not None:
            df = pd.read_csv(io.BytesIO(response))
            return df, ''
        return None, '获取数据失败'

    def get_daily(self, symbol):
        return self._get_data(symbol, 'history')

    def get_dividend(self, symbol):
        return self._get_data(symbol, 'div')

    def get_split(self, symbol):
        return self._get_data(symbol, 'split')



from .wechat_agent import *

agent = WechatMPAgent()


def login(mp_username, mp_password):
    return agent.login(mp_username, mp_password)

def search_pub(pubaccount):
    return agent.search_pub(pubaccount)

def get_total_msg_count(fakeid):
    msg_cnt, df = agent.get_article_list(fakeid, 0)
    return msg_cnt

def get_all_articles(fakeid, app_msg_cnt):
    count = 0
    df_list = []
    while count < app_msg_cnt:
        cnt, df = agent.get_article_list(fakeid, count)
        df_list.append(df)
        count = count + 5
        print(count)
        time.sleep(5)

    result = pd.concat(df_list)
    return result



# encoding: utf-8
from opendatatools.common import RestAgent
from bs4 import BeautifulSoup
import json
import pandas as pd
import datetime
import re


index_map={
    'SSEC'      :   '上证综合指数',
    'SZSC1'     :   '深证成份指数（价格）',
    'FTXIN9'    :   '富时中国A50指数',
    'DJSH'      :   '道琼斯上海指数',
    'HSI'       :   '香港恒生指数 (CFD)',
    'DJI'       :   '道琼斯工业平均指数',
    'SPX'       :   '美国标准普尔500指数 (CFD)',
    'IXIC'      :   '纳斯达克综合指数',
    'RUT'       :   '美国小型股2000 (CFD)',
    'VIX'       :   'VIX恐慌指数 (CFD)',
    'GSPTSE'    :   '加拿大多伦多S&P/TSX 综合指数 (CFD)',
    'BVSP'      :   '巴西IBOVESPA股指',
    'MXX'       :   'S&P/BMV IPC',
    'GDAXI'     :   '德国DAX30指数 (CFD)',
    'FTSE'      :   '英国富时100指数 (CFD)',
    'FCHI'      :   '法国CAC40指数',
    'STOXX50E'  :   '欧洲斯托克(Eurostoxx)50指数 (CFD)',
    'AEX'       :   '荷兰AEX指数',
    'IBEX'      :   '西班牙IBEX35指数 (CFD)',
    'FTMIB'     :   '意大利富时MIB指数 (CFD)',
    'SSMI'      :   '瑞士SWI20指数 (CFD)',
    'PSI20'     :   '葡萄牙PSI20指数',
    'BFX'       :   '比利时BEL20指数 (CFD)',
    'ATX'      :   'ATX',
    'OMXS30'      :   '瑞典OMX斯德哥尔摩30指数',
    'IMOEX'      :   '俄罗斯MOEX Russia指数',
    'IRTS'      :   '俄罗斯交易系统市值加权指数',
    'WIG20'      :   '波兰华沙WIG20指数',
    'BUX'      :   '匈牙利股票交易指数',
    'XU100'      :   '土耳其伊斯坦堡100指数',
    'TA35'      :   'TA 35',
    'TASI'      :   '沙特阿拉伯TASI指数',
    'N225'      :   '日经225指数 (CFD)',
    'AXJO'      :   '澳大利亚S&P/ASX200指数',
    'TWII'      :   '台湾加权指数',
    'SETI'      :   'SET Index',
    'KS11'      :   '韩国KOSPI指数',
    'JKSE'      :   '印尼雅加达综合指数',
    'NSEI'      :   '印度S&P CNX NIFTY指数',
    'BSESN'      :   '印度孟买30指数',
    'HNX30'      :   'HNX 30',
    'CSE'      :   '斯里兰卡科伦坡指数',
    'VIX'      :   'VIX恐慌指数 (CFD)',
    'FTXIN41350': '富时中国A600行业指数 - 化工制品',

}

index_map_inv = {v:k for k, v in index_map.items()}

class YingWeiAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)
        self.add_headers({'Referer': 'https://cn.investing.com/indices/shanghai-composite', 'X-Requested-With': 'XMLHttpRequest'})

    def get_major_index_list(self):
        url = "https://cn.investing.com/indices/major-indices"

        response = self.do_request(url)
        soup = BeautifulSoup(response, "html5lib")
        tables = soup.find_all('table')
        data_list = []
        for table in tables:
            if table.has_attr('id') and table['id'] == 'cr_12':
                trs = table.findAll("tr")
                for tr in trs:
                    if tr.has_attr('id'):
                        tds = tr.findAll('td')
                        time = datetime.datetime.fromtimestamp(int(tds[7]['data-value'])).strftime("%Y-%m-%d %H:%M:%S")
                        data_list.append({'index_name_cn': tr.a['title'],
                                          'index_name':  index_map_inv[tr.a['title']] if tr.a['title'] in index_map_inv else '',
                                          'country' : tds[0].span['title'],
                                          'last': tds[2].text,
                                          'high': tds[3].text,
                                          'low': tds[4].text,
                                          'price_change': tds[5].text,
                                          'percent_change': tds[6].text,
                                          'time' : time,
                                          })
        df = pd.DataFrame(data_list)
        return df, ''

    def _get_symbol(self, link):
        url = "https://cn.investing.com" + link
        response = self.do_request(url)
        soup = BeautifulSoup(response, "html5lib")
        divs = soup.find_all("div")
        for div in divs:
            if div.has_attr("class") and "instrumentHead" in div["class"]:
                text = div.h1.text
                symbol = re.split("[()]", text)[1]
                return symbol

        return ""

    def get_all_index_list(self):
        url = "https://cn.investing.com/indices/world-indices?&majorIndices=on&primarySectors=on&additionalIndices=on&otherIndices=on"
        response = self.do_request(url)
        soup = BeautifulSoup(response, "html5lib")
        tables = soup.find_all('table')
        data = list()
        for table in tables:
            if (table.has_attr("id") and table["id"].startswith("indice_table_")):
                rows = table.find_all("tr")
                for row in rows:
                    cols = row.find_all("td")
                    if len(cols) <= 8:
                        continue
                    id = row["id"][5:]
                    country = cols[0].span["title"]
                    instname = cols[1].a["title"]
                    url = cols[1].a["href"]
                    symbol = self._get_symbol(url)

                    item = {"country": country, "instname": instname, "symbol": symbol, "investing.com.id" : id}
                    print(item)
                    data.append(item)

        df = pd.DataFrame(data)
        return df, ''

    def _get_id(self, symbol):
        url = "https://cn.investing.com/indices/world-indices?&majorIndices=on&primarySectors=on&additionalIndices=on&otherIndices=on"
        response = self.do_request(url)
        soup = BeautifulSoup(response, "html5lib")
        tables = soup.find_all('table')
        for table in tables:
            if table.has_attr('id') and table["id"].startswith("indice_table_"):
                rows = table.findAll("tr")
                for row in rows:
                    if row.has_attr('id'):
                        if row.a['title'] == symbol:
                            return row['id'][5:]
        return None

    def get_index_data(self, symbol, interval, period):
        symbol = index_map[symbol]
        id = self._get_id(symbol)
        if id is None:
            return None, '暂不支持该指数'
        url = "https://cn.investing.com/common/modules/js_instrument_chart/api/data.php"
        param = {
            'pair_id': id,
            'pair_id_for_news': id,
            'chart_type': 'area',
            'pair_interval': interval,
            'candle_count': 120,
            'events': 'yes',
            'volume_series': 'yes',
            'period': period,
        }
        response = self.do_request(url, param=param, encoding='gzip')
        if response is not None:
            jsonobj = json.loads(response)
            df = pd.DataFrame(jsonobj['candles'])
            df.columns = ['time', 'close', '2', '3']
            df = df[['time', 'close']]
            df['time'] = df['time'].apply(lambda x: datetime.datetime.fromtimestamp(int(x) / 1000))
            return df, ''
        else:
            return None, 'error, no data'

    def get_index_data_by_id(self, id):
        url = "https://cn.investing.com/common/modules/js_instrument_chart/api/data.php"
        param = {
            'pair_id': id,
            'pair_id_for_news': id,
            'chart_type': 'area',
            'pair_interval': interval,
            'candle_count': 120,
            'events': 'yes',
            'volume_series': 'yes',
            'period': period,
        }
        response = self.do_request(url, param=param, encoding='gzip')
        if response is not None:
            jsonobj = json.loads(response)
            df = pd.DataFrame(jsonobj['candles'])
            df.columns = ['time', 'close', '2', '3']
            df = df[['time', 'close']]
            df['time'] = df['time'].apply(lambda x: datetime.datetime.fromtimestamp(int(x) / 1000))
            return df, ''
        else:
            return None, 'error, no data'



# encoding: utf-8

from opendatatools.common import RestAgent
import pandas as pd
import json

class EastMoneyAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    # direct： north, south
    def _get_realtime_moneyflow(self, direct):
        url = 'http://ff.eastmoney.com/EM_CapitalFlowInterface/api/js?id=%s&type=EFR&rtntype=2&acces_token=1942f5da9b46b069953c873404aad4b5&js={"data":[(x)]}' % direct
        response = self.do_request(url)
        if response is None:
            return None

        jsonobj = json.loads(response)
        result = []
        for data in jsonobj['data']:
            items = data.split(',')
            if (items[1] == ''):
                break
            result.append({
                'time' : items[0],
                'hgtzj': items[1],
                'sgtzj': items[2],
                'hgtye': items[3],
                'sgtye': items[4],
            })
        return pd.DataFrame(result)

    def get_realtime_moneyflow(self):
        directs = ['north', 'south']
        df_data = []
        for direct in directs:
            df = self._get_realtime_moneyflow(direct)
            df['direct'] = direct
            if df is not None:
                df_data.append(df)

        if len(df_data) > 0:
            return pd.concat(df_data), ''

        return None, '获取数据失败'

    def get_hist_moneyflow(self):
        url = 'http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get?type=HSGTZJZS&token=70f12f2f4f091e459a279469fe49eca5&js={"data":(x)}'
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        jsonobj = json.loads(response)
        df = pd.DataFrame(jsonobj['data'])
        return df, ''

    def _get_his_tradestat_onepage(self, markettype, page_no):
        url = 'http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get?type=HSGTHIS&token=70f12f2f4f091e459a279469fe49eca5&filter=(MarketType=%d)&js={"data":(x)}&ps=500&p=%d' % (markettype, page_no)
        response = self.do_request(url)
        if response is None:
            return None

        jsonobj = json.loads(response)
        return pd.DataFrame(jsonobj['data'])

    def get_his_tradestat(self, markettype):
        page_no = 1
        df_data = []
        while True:
            df = self._get_his_tradestat_onepage(markettype, page_no)
            if df is None:
                return None, '获取数据失败'
            if len(df) > 0:
                df_data.append(df)

            if len(df) < 500:
                break

            page_no = page_no + 1

        if len(df_data) == 0:
            return None, '获取数据失败'

        return pd.concat(df_data), ''

    def get_ah_compare(self):
        url = 'http://nufm.dfcfw.com/EM_Finance2014NumericApplication/JS.aspx?type=CT&token=4f1862fc3b5e77c150a2b985b12db0fd&p=1&ps=1000&sty=FCABHL&cmd=C._AHH&js={"data":[(x)]}'
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        jsonobj = json.loads(response)
        result = []
        for data in jsonobj['data']:
            items = data.split(',')
            result.append({
                "hshare_code" : items[1],
                'hshare_name' : items[2],
                'hshare_last' : items[3],
                'hshare_chg'  : items[4],
                "ashare_code" : items[5],
                'ashare_name' : items[7],
                'ashare_last' : items[8],
                'ashare_chg'  : items[9],
                'ah_ratio'    : items[13],
            })
        df = pd.DataFrame(result)
        return df, ''


from opendatatools.common import RestAgent
import hashlib
import json
from PIL import Image
from io import BytesIO
import random
import time
import urllib
import pandas as pd
import math
import datetime
import threading
import functools


Host = "mp.weixin.qq.com"
agent = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36"
refer = "https://mp.weixin.qq.com"
xrw = "XMLHttpRequest"

loginUrl = "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=startlogin"
qrcodeUrl = "https://mp.weixin.qq.com/cgi-bin/loginqrcode?action=getqrcode&param=4300&rd=120"
checkLogin = "https://mp.weixin.qq.com/cgi-bin/loginqrcode?action=ask&f=json&ajax=1&random="
doLogin = "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=login"
searchUrl = "https://mp.weixin.qq.com/cgi-bin/searchbiz?action=search_biz&needToken&lang=zh_CN&f=json&ajax=1&needRandom&needQuery&begin=0&count=5"
appmsg = "https://mp.weixin.qq.com/cgi-bin/appmsg?needToken&lang=zh_CN&f=json&ajax=1&needRandom&action=list_ex&needBegin&needCount&query=&needFakeid&type=9" # 原9,改为1后条数变多

def md5(data):
    m = hashlib.md5()
    m.update(data.encode(encoding='UTF-8'))
    return m.hexdigest()


def ReqRandom():
    ll = random.random() * 100000000000000000
    result = "0." + str(ll)
    return result

class WechatMPAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def login(self, username, password):
        param = {
            "username": username,
            "pwd": md5(password),
            "imgcode": "",
            "f": "json"
        }

        self.add_headers({
            "Referer" : "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=validate&lang=zh_CN&account=PKUJohnson@163.com",
            "Host" : Host,
            "X-Requested-With" : xrw
        })

        resp = self.do_request(loginUrl, method="POST", param=param)
        print(resp)
        redirect_url = json.loads(resp)["redirect_url"]

        response = self.do_request(qrcodeUrl, method="GET", type="binary")
        qrcode = Image.open(BytesIO(response))
        qrcode.show()

        # Check if qrcode is verified
        while True:
            response = self.do_request(checkLogin+ReqRandom(), method="GET")
            rsp = json.loads(response)

            if rsp["status"] == 1:
                qrcode.close()
                break
            else:
                time.sleep(2)

        param = {
            "userlang": "zh_CN",
            "token": "",
            "lang": "zh_CN",
            "f" : "json",
            "ajax" : "1",

        }

        response = self.do_request(doLogin, method="POST", param=param)
        rsp = json.loads(response)
        print(rsp)
        token_url = rsp["redirect_url"]
        pos = token_url.find("token=")
        token = token_url[pos+6:]
        self.token = token
        return True

    def get_retcode(self, rsp):
        if "base_resp" in rsp and "ret" in rsp["base_resp"] and "err_msg" in rsp["base_resp"]:
            return rsp["base_resp"]["ret"], rsp["base_resp"]["err_msg"]
        return 0, "ok"

    def search_pub(self, pubno):
        _searchUrl = searchUrl.replace("needToken", "token=" + self.token).replace("needRandom", "random=" + ReqRandom()).replace("needQuery", "query=" + pubno)
        response = self.do_request(_searchUrl, method="GET")
        rsp = json.loads(response)
        ret, err_msg = self.get_retcode(rsp)
        if ret != 0:
            return None, err_msg
        df = pd.DataFrame(rsp["list"])
        return df, ""

    def get_article_list(self, fakeid, begin):
        count = 5
        _appmsg = appmsg.replace("needToken", "token=" + self.token).replace("needRandom", "random=" + ReqRandom()).replace("needFakeid", "fakeid=" + fakeid)
        appmsgTemp = _appmsg.replace("needBegin", "begin=" + str(begin)).replace("needCount", "count=" + str(count))

        response = self.do_request(appmsgTemp, method="GET")
        rsp = json.loads(response)
        ret, msg = self.get_retcode(rsp)
        # 失败后60秒再试一次
        while msg != "ok":
            print(response.text)
            time.sleep(60)
            response = self.do_request(appmsgTemp, method="GET")
            rsp = json.loads(response.text)
            ret, msg = self.get_retcode(rsp)

        app_msg_cnt = rsp["app_msg_cnt"]
        app_msg_list = rsp["app_msg_list"]
        df = pd.DataFrame(app_msg_list)
        return app_msg_cnt, df




# encoding: utf-8
from opendatatools import fx
import unittest


class TestFx(unittest.TestCase):
    def test_get_his_cny_cpr(self):
        df, msg = fx.get_hist_cny_cpr()
        assert (len(df) >= 100)

        df, msg = fx.get_hist_cny_cpr('2017-01-01', '2018-05-01')
        assert (df == None)



# encoding: utf-8

import datetime
import pandas as pd
from bs4 import BeautifulSoup
from xml.etree import ElementTree

from opendatatools.common import RestAgent

class SWIndexAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def get_index_list(self):
        url = 'http://www.swsindex.com/idx0110.aspx'

        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        soup = BeautifulSoup(response, "html5lib")

        sections = soup.find_all("div", class_="honor")

        section_names = [
            '量化策略', '市场表征', '一级行业', '风格指数', '申万商品期货指数', '新三板', '定制发布'
        ]

        data = []
        for section in sections:
            name = section.h3.text.strip()
            if name not in section_names:
                continue

            table = section.table
            rows = table.findAll('tr')
            for row in rows:
                cols = row.findAll('td')
                if (len(cols) >= 4):
                    index_name = cols[0].text
                    start_date = cols[1].text
                    index_code = cols[0].a['href'][-6:]

                    data.append({
                        'index_code'   : index_code,
                        'index_name'   : index_name,
                        'start_date'   : start_date,
                        'section_name' : name
                    })
        return pd.DataFrame(data), ''

    def get_index_cons(self, index_code):
        url = 'http://www.swsindex.com/downfile.aspx?code=%s' %  index_code
        response = self.do_request(url, None, method='GET', type='text')
        if response is not None:
            soup = BeautifulSoup(response, "html5lib")

            data = []
            table = soup.findAll('table')[0]
            rows = table.findAll('tr')
            for row in rows:
                cols = row.findAll('td')
                if (len(cols) >= 4):
                    stock_code = cols[0].text
                    stock_name = cols[1].text
                    weight     = cols[2].text
                    start_date = cols[3].text

                    data.append({
                        'stock_code': stock_code,
                        'stock_name': stock_name,
                        'start_date': start_date,
                        'weight'     : weight
                    })
            df = pd.DataFrame(data)
            if len(df) > 0:
                df['start_date'] = df['start_date'].apply(lambda x : datetime.datetime.strptime(x, '%Y/%m/%d %H:%M:%S'))
            return df, ''

        return None, '获取数据失败'

    def get_index_daily(self, index_code, start_date, end_date):
        url = 'http://www.swsindex.com/excel2.aspx?ctable=swindexhistory&where=%s '
        where_cond = " swindexcode in ('%s') and BargainDate >= '%s' and BargainDate <= '%s'" % (index_code, start_date, end_date)
        url = url % where_cond
        print(url)

        response = self.do_request(url, None, method='GET', type='text', encoding='utf8')
        if response is None:
            return None, '获取数据失败'

        soup = BeautifulSoup(response, "html5lib")
        data = []
        table = soup.findAll('table')[0]
        rows = table.findAll('tr')
        for row in rows:
            cols = row.findAll('td')
            if (len(cols) >= 10):
                index_code = cols[0].text
                index_name = cols[1].text
                date = cols[2].text
                open = cols[3].text
                high = cols[4].text
                low = cols[5].text
                close = cols[6].text
                vol = cols[7].text
                amount = cols[8].text
                change_pct = cols[9].text

                data.append({
                    'index_code': index_code.replace(",", ""),
                    'index_name': index_name.replace(",", ""),
                    'date': date.replace(",", ""),
                    'open': open.replace(",", ""),
                    'high': high.replace(",", ""),
                    'low': low.replace(",", ""),
                    'close': close.replace(",", ""),
                    'vol': vol.replace(",", ""),
                    'amount': amount.replace(",", ""),
                    'change_pct': change_pct.replace(",", ""),
                })

        df = pd.DataFrame(data)
        if len(df) > 0:
            df['date'] = df['date'].apply(lambda x: datetime.datetime.strptime(x, '%Y/%m/%d %H:%M:%S'))
        return df, ''

    def get_index_dailyindicator(self, index_code, start_date, end_date, type):
        url = 'http://www.swsindex.com/excel.aspx?ctable=V_Report&where=%s'
        where_cond = "swindexcode in ('%s') and BargainDate >= '%s' and BargainDate <= '%s' and type='%s'" % (index_code, start_date, end_date, type)
        url = url % where_cond

        response = self.do_request(url, None, method='GET', type='text', encoding='utf8')
        if response is None:
            return None, '获取数据失败'

        soup = BeautifulSoup(response, "html5lib")
        data = []
        table = soup.findAll('table')[0]
        rows = table.findAll('tr')
        for row in rows:
            cols = row.findAll('td')
            if (len(cols) >= 14):
                index_code = cols[0].text
                index_name = cols[1].text
                date = cols[2].text
                close = cols[3].text
                volume = cols[4].text
                chg_pct = cols[5].text
                turn_rate = cols[6].text
                pe = cols[7].text
                pb = cols[8].text
                vwap = cols[9].text
                turnover_pct = cols[10].text
                float_mv = cols[11].text
                avg_float_mv = cols[12].text
                dividend_yield_ratio = cols[13].text

                data.append({
                    'index_code': index_code,
                    'index_name': index_name,
                    'date': date,
                    'close': close,
                    'volume': volume,
                    'chg_pct': chg_pct,
                    'turn_rate': turn_rate,
                    'pe': pe,
                    'pb': pb,
                    'vwap': vwap,
                    'float_mv': float_mv,
                    'avg_float_mv': avg_float_mv,
                    'dividend_yield_ratio': dividend_yield_ratio,
                    'turnover_pct' : turnover_pct
                })

        df = pd.DataFrame(data)
        if len(df)>0:
            df['date'] = df['date'].apply(lambda x: datetime.datetime.strptime(x, '%Y/%m/%d %H:%M:%S'))
        return df, ''



# encoding: utf-8
from opendatatools import futures
import unittest


class TestFutures(unittest.TestCase):
    def test_get_trade_rank(self):
        markets = ['SHF', 'CZC', 'DCE', 'CFE']
        for market in markets:
            df, msg = futures.get_trade_rank(market)
            assert (len(df) >= 100)



# encoding: utf-8

from opendatatools.common import RestAgent
from progressbar import ProgressBar
import demjson
import json
import pandas as pd

fund_type = {
    "全部开放基金" :  {"t": 1, "lx": 1},
    "股票型基金"   :  {"t": 1, "lx": 2},
    "混合型基金"   :  {"t": 1, "lx": 3},
    "债券型基金"   :  {"t": 1, "lx": 4},
    "指数型基金"   :  {"t": 1, "lx": 5},
    "ETF联接基金"  :  {"t": 1, "lx": 6},
    "LOF基金"      :  {"t": 1, "lx": 8},
    "分级基金"     :  {"t": 1, "lx": 9},
    "FOF基金"      :  {"t": 1, "lx": 15},
    "理财基金"     :  {"t": 5},
    "分级A"        :  {"t": 6},
    "货币基金"     : {"t": 7},
}

class EastMoneyAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def _get_and_parse_js(self, url, prefix, param=None):
        response = self.do_request(url, param=param)
        if not response.startswith(prefix):
            return None
        else:
            return response[len(prefix):]

    def get_fund_company(self):
        url    = 'http://fund.eastmoney.com/Data/Fund_JJJZ_Data.aspx?t=3'
        prefix = 'var gs='
        response = self._get_and_parse_js(url, prefix)
        if response is None:
            return None, '获取数据失败'

        jsonobj = demjson.decode(response)
        df = pd.DataFrame(jsonobj['op'])
        df.columns = ['companyid', 'companyname']
        return df, ''

    def _get_fund_list_onepage(self, company='', page_no = 1, page_size = 100):
        url    = 'http://fund.eastmoney.com/Data/Fund_JJJZ_Data.aspx?page=%d,%d&gsid=%s' % (page_no, page_size, company)
        prefix = 'var db='
        response = self.do_request(url)
        response = self._get_and_parse_js(url, prefix)
        if response is None:
            return None, '获取数据失败'

        jsonobj = demjson.decode(response)
        rsp = jsonobj['datas']
        datestr = jsonobj['showday']
        df = pd.DataFrame(rsp)
        if len(df) > 0:
            df.drop(df.columns[5:], axis=1, inplace=True)
            df.columns = ['fundcode', 'fundname', 'pingyin', 'nav', 'accu_nav']
            df['date'] = datestr[0]
            return df, ''
        else:
            return None, ''

    def get_fundlist_by_company(self, companyid):
        page_no = 1
        page_size = 1000

        df_result = []
        while True:
            df, msg = self._get_fund_list_onepage(company=companyid, page_no=page_no, page_size=page_size)
            if df is not None:
                df_result.append(df)

            if df is None or len(df) < page_size:
                break
            page_no = page_no + 1

        if len(df_result) > 0:
            return pd.concat(df_result), ''
        else:
            return None, ''

    def get_fund_list(self):
        df_company, msg = self.get_fund_company()
        if df_company is None:
            return None, msg

        df_result = []
        process_bar = ProgressBar().start(max_value=len(df_company))
        for index, row in df_company.iterrows():
            companyid   = row['companyid']
            companyname = row['companyname']

            df, msg = self.get_fundlist_by_company(companyid)
            if df is not None:
                df['companyname'] = companyname
                df['companyid']   = companyid
                df_result.append(df)

            process_bar.update(index+1)

        return pd.concat(df_result), ''

    def get_fund_type(self):
        return fund_type.keys()

    def _get_fundlist_by_type_page(self, type, page_no = 1, page_size = 100):
        url    = 'http://fund.eastmoney.com/Data/Fund_JJJZ_Data.aspx?page=%d,%d' % (page_no, page_size)
        prefix = 'var db='
        type_param = fund_type[type]

        response = self._get_and_parse_js(url, prefix, param=type_param)
        jsonobj = demjson.decode(response)
        rsp = jsonobj['datas']
        datestr = jsonobj['showday']
        df = pd.DataFrame(rsp)
        if len(df) > 0:
            df.drop(df.columns[5:], axis=1, inplace=True)
            df.columns = ['fundcode', 'fundname', 'pingyin', 'nav', 'accu_nav']
            df['date'] = datestr[0]
            return df, ''
        else:
            return None, '获取数据失败'

    def get_fundlist_by_type(self, type):
        if type not in fund_type:
            return None, '不正确的基金类型，请通过get_fund_type查询'
        type_param = fund_type[type]

        page_no = 1
        page_size = 1000
        df_result = []
        while True:
            df, msg = self._get_fundlist_by_type_page(type, page_no, page_size)
            if df is not None:
                df_result.append(df)

            if df is None or len(df)< page_size:
                break

            page_no = page_no + 1

        df = pd.concat(df_result)
        df['fund_type'] = type
        return df, ''

    def get_fund_nav(self, fund_code):
        url = 'http://api.fund.eastmoney.com/f10/lsjz'
        self.add_headers({'Referer': 'http://fund.eastmoney.com/f10/jjjz_%s.html' % fund_code})

        page_no   = 1
        page_size = 1000
        df_result = []
        while True:
            data = {
                'fundCode' : fund_code,
                'pageIndex': page_no,
                'pageSize' : page_size,
            }

            response = self.do_request(url, param=data)
            jsonobj = json.loads(response)
            err_code = jsonobj['ErrCode']
            err_msg  = jsonobj['ErrMsg']

            if err_code != 0:
                return None, err_msg

            rsp = jsonobj['Data']['LSJZList']
            df = pd.DataFrame(rsp)
            if len(df) > 0 :
                df_result.append(df)

            if len(df) < page_size:
                break

            page_no = page_no + 1

        df = pd.concat(df_result)
        df_ret = df[['FSRQ', 'DWJZ', 'LJJZ']]
        df_ret.columns = ['date', 'nav1', 'nav2']
        return df_ret, ''


# encoding = 'utf-8'
from .index_agent import *

agent = YingWeiAgent()

time_map = {
    '1h' : 3600,
    '1d' : 86400,
    '1w' : 'week',
    '1m' : 'month',
    '3m' : '3-months',
    '6m' : '6-months',
    '1y' : '1-year',
    '5y' : '5-years',
    'max': 'max',
}


def get_index_data(symbol, freq, period):
    period = time_map[period]
    freq = time_map[freq]
    return agent.get_index_data(symbol=symbol, period=period, interval=freq)


def get_major_index_list():
    df, msg = agent.get_major_index_list()
    return df, msg

def get_all_index_list():
    df, msg = agent.get_all_index_list()
    return df, msg

def set_proxies(proxies):
    return agent.set_proxies(proxies)




